from collections import namedtuple

class ShapeFilter(namedtuple("ShapeFilter", ['group', 'categories', 'mask'])):
    """
    Pymunk has two primary means of ignoring collisions: groups and 
    category masks.

    Groups are used to ignore collisions between parts on a complex object. A 
    ragdoll is a good example. When jointing an arm onto the torso, you'll 
    want them to allow them to overlap. Groups allow you to do exactly that. 
    Shapes that have the same group don't generate collisions. So by placing 
    all of the shapes in a ragdoll in the same group, you'll prevent it from 
    colliding against other parts of itself. Category masks allow you to mark 
    which categories an object belongs to and which categories it collides 
    with.

    For example, a game has four collision categories: player (0), enemy (1), 
    player bullet (2), and enemy bullet (3). Neither players nor enemies 
    should not collide with their own bullets, and bullets should not collide 
    with other bullets. However, players collide with enemy bullets, and 
    enemies collide with player bullets.

    ============= =============== ====================
    Object        Object Category Category Mask
    ============= =============== ====================
    Player        0b00001 (1)     0b11000 (4, 5)       
    Enemy         0b00010 (2)     0b01110 (2, 3, 4)
    Player Bullet 0b00100 (3)     0b10001 (1, 5)
    Enemy Bullet  0b01000 (4)     0b10010 (2, 5)
    Walls         0b10000 (5)     0b01111 (1, 2, 3, 4)
    ============= =============== ====================
    
    Note that in the table the categories and masks are written as binary 
    values to clearly show the logic. To save space only 5 digits are used. The 
    default type of categories and mask in ShapeFilter is an unsigned int, 
    with a resolution of 32 bits. That means that the you have 32 bits to use, 
    in binary notation that is `0b00000000000000000000000000000000` to 
    `0b11111111111111111111111111111111` which can be written in hex as 
    `0x00000000` to `0xFFFFFFFF`.

    Everything in this example collides with walls. Additionally, 
    the enemies collide with each other.

    By default, objects exist in every category and collide with every category.

    Objects can fall into multiple categories. For instance, you might have a 
    category for a red team, and have a red player bullet. In the above 
    example, each object only has one category. 

    The default type of categories and mask in ShapeFilter is unsigned int 
    which has a resolution of 32 bits on most systems. 

    There is one last way of filtering collisions using collision handlers. 
    See the section on callbacks for more information. Collision handlers can 
    be more flexible, but can be slower. Fast collision filtering rejects 
    collisions before running the expensive collision detection code, so 
    using groups or category masks is preferred.

    Example of how category and mask can be used to filter out player from 
    enemy object:

    >>> import pymunk
    >>> s = pymunk.Space()
    >>> player_b = pymunk.Body(1,1)
    >>> player_c = pymunk.Circle(player_b, 10)
    >>> s.add(player_b, player_c)
    >>> player_c.filter = pymunk.ShapeFilter(categories=0b1)
    >>> hit = s.point_query_nearest((0,0), 0, pymunk.ShapeFilter())
    >>> hit != None
    True
    >>> filter = pymunk.ShapeFilter(mask=pymunk.ShapeFilter.ALL_MASKS ^ 0b1)
    >>> hit = s.point_query_nearest((0,0), 0, filter)
    >>> hit == None
    True
    >>> enemy_b = pymunk.Body(1,1)
    >>> enemy_c = pymunk.Circle(enemy_b, 10)
    >>> s.add(enemy_b, enemy_c)
    >>> hit = s.point_query_nearest((0,0), 0, filter)
    >>> hit != None
    True

    """
    __slots__ = ()    

    ALL_CATEGORIES = 0xffffffff

    ALL_MASKS = 0xffffffff

    def __new__(cls, group = 0, categories = 0xffffffff, mask = 0xffffffff):
        self = super(ShapeFilter, cls).__new__(cls, group, categories, mask)
        return self


import pymunk as p
from pymunk.vec2d import Vec2d
import unittest
import pickle

####################################################################

class UnitTestBody(unittest.TestCase):
    def testProperties(self):
        b = p.Body(10,100)
        self.assertEqual(b.mass, 10)
        b.mass = 11
        self.assertEqual(b.mass, 11)

        self.assertEqual(b.moment, 100)
        b.moment = 101
        self.assertEqual(b.moment, 101)

        self.assertEqual(b.position, (0,0))
        b.position = 1,2
        self.assertEqual(b.position, (1,2))

        self.assertEqual(b.center_of_gravity, (0,0))
        b.center_of_gravity = 2,3
        self.assertEqual(b.center_of_gravity, (2,3))

        self.assertEqual(b.velocity, (0,0))
        b.velocity = 3,4
        self.assertEqual(b.velocity, (3,4))

        self.assertEqual(b.force, (0,0))
        b.force = (4,5)
        self.assertEqual(b.force, (4,5))

        self.assertEqual(b.angle, 0)
        b.angle = 1.2
        self.assertEqual(b.angle, 1.2)

        self.assertEqual(b.angular_velocity, 0)
        b.angular_velocity = 1.3
        self.assertEqual(b.angular_velocity, 1.3)

        self.assertEqual(b.torque, 0)
        b.torque = 1.4
        self.assertEqual(b.torque, 1.4)

        b.angle = 0
        self.assertEqual(b.rotation_vector, (1,0))
        b.angle = Vec2d(1,1).get_angle()
        self.assertAlmostEqual(b.rotation_vector.get_angle(), Vec2d(1,1).get_angle())

        self.assertEqual(b.space, None)
        s = p.Space()
        s.add(b)
        self.assertEqual(b.space, s)

    def testRepr(self):
        b = p.Body(1,2)
        self.assertEqual(str(b), "Body(1.0, 2.0, Body.DYNAMIC)")
        b = p.Body(body_type = p.Body.KINEMATIC)
        self.assertEqual(str(b), "Body(Body.KINEMATIC)")
        b = p.Body(body_type = p.Body.STATIC)
        self.assertEqual(str(b), "Body(Body.STATIC)")

    def testCoordinateConversion(self):
        b = p.Body(body_type=p.Body.KINEMATIC)
        v = 1,2
        self.assertEqual(b.local_to_world(v), v)
        self.assertEqual(b.world_to_local(v), v)
        b.position = 3,4
        self.assertEqual(b.local_to_world(v), (4,6))
        self.assertEqual(b.world_to_local(v), (-2,-2))

    def testVelocityConversion(self):
        b = p.Body(1,2)
        self.assertEqual(b.velocity_at_world_point((1,1)), (0,0))
        self.assertEqual(b.velocity_at_local_point((1,1)), (0,0))
        b.position = 1,2
        b.angular_velocity = 1.2
        self.assertEqual(b.velocity_at_world_point((1,1)), (1.2, 0))
        self.assertEqual(b.velocity_at_local_point((1,1)), (-1.2, 1.2))

    def testForce(self):
        b = p.Body(1,2)
        b.position = 3,4
        b.apply_force_at_world_point((10,0), (0,10))
        self.assertEqual(b.force, (10,0))
        self.assertEqual(b.torque, -60)

        b = p.Body(1,2)
        b.position = 3,4
        b.apply_force_at_local_point((10,0), (0,10))
        self.assertEqual(b.force, (10,0))
        self.assertEqual(b.torque, -100)

    def testImpulse(self):
        b = p.Body(1,2)
        b.position = 3,4
        b.apply_impulse_at_world_point((10,0), (0,10))
        self.assertEqual(b.velocity, (10,0))
        self.assertEqual(b.angular_velocity, -30)

        b = p.Body(1,2)
        b.position = 3,4
        b.apply_impulse_at_local_point((10,0), (0,10))
        self.assertEqual(b.velocity, (10,0))
        self.assertEqual(b.angular_velocity, -50)

    def testSleep(self):
        b = p.Body(1,1)
        s = p.Space()
        s.sleep_time_threshold = 0.01
        
        self.assertFalse(b.is_sleeping)

        self.assertRaises(Exception, b.sleep)
        s.add(b)
        b.sleep()

        self.assertTrue(b.is_sleeping)

        b.activate()
        self.assertFalse(b.is_sleeping)

        b.sleep()
        s.remove(b)
        b.activate()
        

    def testSleepWithGroup(self):
        b1 = p.Body(1,1)
        b2 = p.Body(2,2)
        s = p.Space()
        s.sleep_time_threshold = 0.01
        s.add(b2)
        b2.sleep()
        
        with self.assertRaises(Exception):
            b1.sleep_with_group(b2)
        
        s.add(b1)
        b1.sleep_with_group(b2)
        self.assertTrue(b1.is_sleeping)
        b2.activate()
        self.assertFalse(b1.is_sleeping)


    def testKineticEnergy(self):
        b = p.Body(1,10)
        self.assertEqual(b.kinetic_energy, 0)
        b.apply_impulse_at_local_point((10,0))
        self.assertEqual(b.kinetic_energy, 100)

    def testDynamic(self):
        b1 = p.Body(1,1)
        b2 = p.Body(1,1, body_type=p.Body.DYNAMIC)
        self.assertEqual(b1.body_type, p.Body.DYNAMIC)
        self.assertEqual(b2.body_type, p.Body.DYNAMIC)
       
    def testKinematic(self):
        b = p.Body(body_type=p.Body.KINEMATIC)
        self.assertEqual(b.body_type, p.Body.KINEMATIC)

    def testStatic(self):
        b = p.Body(body_type=p.Body.STATIC)
        self.assertEqual(b.body_type, p.Body.STATIC)

    def testMassMomentFromShape(self):
        s = p.Space()
        
        b = p.Body()
        b.mass = 2
        c = p.Circle(b, 10, (2,3))
        c.mass = 3
        
        self.assertEqual(b.mass, 0)
        s.add(b, c)
        self.assertEqual(b.mass, 3)
        c.mass = 4
        self.assertEqual(b.mass, 4)
        self.assertEqual(b.center_of_gravity.x, 2)
        self.assertEqual(b.center_of_gravity.y, 3)
        self.assertEqual(b.moment, 200)

    def testPositionFunction(self):
        s = p.Space()
        b = p.Body(1,1)
        def f(body, dt):
            body.position += 0,dt

        b.position_func=f
        s.add(b)
        s.step(10)
        self.assertEqual(b.position, (0,10))
        s.step(1)
        s.step(1)
        self.assertEqual(b.position.y, 12)

        b.position_func = p.Body.update_position
        s.step(1)
        self.assertEqual(b.position.y, 12)

    def testVelocityFunction(self):
        s = p.Space()
        b = p.Body(1,1)
        def f(body, gravity, damping, dt):
            body.velocity += 5*gravity

        b.velocity_func=f
        s.gravity = 1,0
        s.add(b)
        s.step(10)
        return
        self.assertEqual(b.velocity.x, 5)
        s.step(0.1)
        s.step(0.1)
        self.assertEqual(b.velocity.x, 15)

        b.velocity_func = b.update_velocity
        s.step(1)
        self.assertEqual(b.velocity.x, 16)

    def testEachArbiters(self):
        s = p.Space()
        b1 = p.Body(1,1)
        b2 = p.Body(1,1)
        c1 = p.Circle(b1,10)
        c2 = p.Circle(b2,10)
        s.add(b1,b2,c1,c2)
        s.step(1)

        shapes = []
        def f(arbiter, shapes):
            shapes += arbiter.shapes
        arbs = b1.each_arbiter(f, shapes)
        self.assertEqual(shapes[0], c1)
        self.assertEqual(shapes[1], c2)

    def testGetConstraints(self):
        s = p.Space()
        b1 = p.Body(1,1)
        b2 = p.Body(1,1)
        s.add(b1)
        j1 = p.PivotJoint(b1,s.static_body,(0,0))
        j2 = p.PivotJoint(b2,s.static_body,(0,0))

        self.assertTrue(j1 in b1.constraints)
        self.assertTrue(j1 not in b2.constraints)
        self.assertTrue(j1 in s.static_body.constraints)
        self.assertTrue(j2 in s.static_body.constraints)

    def testGetShapes(self):
        s = p.Space()
        b1 = p.Body(1,1)
        s.add(b1)
        s1 = p.Circle(b1,3)
        s2 = p.Segment(b1,(0,0), (1,2),1)

        self.assertTrue(s1 in b1.shapes)
        self.assertTrue(s2 in b1.shapes)
        self.assertTrue(s1 not in s.static_body.shapes)


    def testPickle(self):
        b = p.Body(1, 2)
        b.custom = "test"
        b.position = 3,4
        b.center_of_gravity = 5,6
        b.velocity = 7,8
        b.force = 9,10
        b.angle = 11
        b.angular_velocity = 12
        b.torque = 13
        
        b.position_func = pf
        b.velocity_func = vf
        print(1)
        s = pickle.dumps(b)
        print(2)
        b2 = pickle.loads(s)
        print(3)
        self.assertEqual(b.mass, b2.mass)
        self.assertEqual(b.moment, b2.moment)
        self.assertEqual(b.body_type, b2.body_type)
        self.assertEqual(b.custom, b2.custom)
        self.assertEqual(b.position, b2.position)
        self.assertEqual(b.center_of_gravity, b2.center_of_gravity)
        self.assertEqual(b.velocity, b2.velocity)
        self.assertEqual(b.force, b2.force)
        self.assertEqual(b.angle, b2.angle)
        self.assertEqual(b.angular_velocity, b2.angular_velocity)
        self.assertEqual(b.torque, b2.torque)
        
        space = p.Space()
        space.add(b2)
        print(4)
        space.step(0.1)
        
        self.assertTrue(b2.pf)
        self.assertTrue(b2.vf)
        print(5)
        b2 = b.copy()
        print(6)

# Needs to be here for the lowest pickle protocol to work    
def pf(body, dt):
    body.pf = True

def vf(body, gravity, damping, dt):
    body.vf = True
    

####################################################################
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()



from __future__ import with_statement
import sys, io
import unittest
import warnings
import pickle
import copy

from pymunk import *
import pymunk as p
from pymunk.vec2d import Vec2d

####################################################################

class UnitTestSpace(unittest.TestCase):
    def setUp(self):
        self.s = p.Space()

        self.b1, self.b2 = p.Body(1,3),p.Body(10,100)
        self.s.add(self.b1,self.b2)
        self.b1.position = 10,0
        self.b2.position = 20,0

        self.s1,self.s2 = p.Circle(self.b1,5), p.Circle(self.b2,10)
        self.s.add(self.s1,self.s2)

    def tearDown(self):
        del self.s
        del self.b1, self.b2
        del self.s1, self.s2

    def testProperties(self):
        s = p.Space()

        self.assertEqual(s.iterations, 10)
        s.iterations = 15
        self.assertEqual(s.iterations, 15)

        self.assertEqual(s.gravity, (0,0))
        s.gravity = 10,2
        self.assertEqual(s.gravity, (10,2))
        self.assertEqual(s.gravity.x, 10)

        self.assertEqual(s.damping, 1)
        s.damping = 3
        self.assertEqual(s.damping, 3)

        self.assertEqual(s.idle_speed_threshold, 0)
        s.idle_speed_threshold = 4
        self.assertEqual(s.idle_speed_threshold, 4)

        self.assertEqual(str(s.sleep_time_threshold), 'inf')
        s.sleep_time_threshold = 5
        self.assertEqual(s.sleep_time_threshold, 5)

        self.assertAlmostEqual(s.collision_slop, 0.1)
        s.collision_slop = 6
        self.assertEqual(s.collision_slop, 6)

        self.assertAlmostEqual(s.collision_bias, 0.0017970074436)
        s.collision_bias = 0.2
        self.assertEqual(s.collision_bias, 0.2)

        self.assertEqual(s.collision_persistence, 3)
        s.collision_persistence = 9
        self.assertEqual(s.collision_persistence, 9)

        self.assertEqual(s.current_time_step, 0)
        s.step(0.1)
        self.assertEqual(s.current_time_step, 0.1)

        self.assertTrue(s.static_body != None)
        self.assertEqual(s.static_body.body_type, p.Body.STATIC)
        
        self.assertEqual(s.threads, 1)
        s.threads = 2
        self.assertEqual(s.threads, 1)


    def testThreaded(self):
        s = p.Space(threaded=True)
        s.step(1)
        s.threads = 2
        import platform
        if platform.system() == 'Windows':
            self.assertEqual(s.threads, 1)
        else:
            self.assertEqual(s.threads, 2)
        s.step(1)

    def testAddRemove(self):
        s = p.Space()

        self.assertEqual(s.bodies, [])
        self.assertEqual(s.shapes, [])

        b = p.Body(1,2)
        s.add(b)
        self.assertEqual(s.bodies, [b])
        self.assertEqual(s.shapes, [])

        c1 = p.Circle(b, 10)
        s.add(c1)
        self.assertEqual(s.bodies, [b])
        self.assertEqual(s.shapes, [c1])

        c2 = p.Circle(b, 15)
        s.add(c2)
        self.assertEqual(len(s.shapes), 2)
        self.assertTrue(c1 in s.shapes)
        self.assertTrue(c2 in s.shapes)
        
        s.remove(c1)
        self.assertEqual(s.shapes, [c2])

        s.remove(c2, b)
        self.assertEqual(s.bodies, [])
        self.assertEqual(s.shapes, [])

        s.add(c2, b)
        self.assertEqual(s.bodies, [b])
        self.assertEqual(s.shapes, [c2])

    def testAddRemoveInStep(self):
        s = p.Space()

        b1 = p.Body(1, 2)
        c1 = p.Circle(b1, 2)

        b2 = p.Body(1, 2)
        c2 = p.Circle(b2, 2)

        s.add(b1, b2, c1, c2)

        b = p.Body(1, 2)
        c = p.Circle(b, 2)
        
        def pre_solve_add(arbiter, space, data):
            space.add(b, c)
            space.add(c, b)
            self.assertTrue(b not in s.bodies)
            self.assertTrue(c not in s.shapes)
            return True

        def pre_solve_remove(arbiter, space, data):
            space.remove(b, c)
            space.remove(c, b)
            self.assertTrue(b in s.bodies)
            self.assertTrue(c in s.shapes)
            return True

        s.add_collision_handler(0, 0).pre_solve = pre_solve_add

        s.step(.1)
        
        self.assertTrue(b in s.bodies)
        self.assertTrue(c in s.shapes)

        s.add_collision_handler(0, 0).pre_solve = pre_solve_remove
        
        s.step(.1)
        
        self.assertTrue(b not in s.bodies)
        self.assertTrue(c not in s.shapes)

    def testRemoveInStep(self):
        s = self.s

        def pre_solve(arbiter, space, data):
            space.remove(arbiter.shapes)
            return True

        s.add_collision_handler(0, 0).pre_solve = pre_solve

        s.step(.1)

        self.assertTrue(self.s1 not in s.bodies)
        self.assertTrue(self.s2 not in s.shapes)

    def testPointQueryNearestWithShapeFilter(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        s1 = p.Circle(b1, 10)
        s.add(s1)

        tests = [
            {"c1": 0b00, "m1":0b00, "c2": 0b00, "m2":0b00, "hit":0}, 
            {"c1": 0b01, "m1":0b01, "c2": 0b01, "m2":0b01, "hit":1}, 
            {"c1": 0b10, "m1":0b01, "c2": 0b01, "m2":0b10, "hit":1}, 
            {"c1": 0b01, "m1":0b01, "c2": 0b11, "m2":0b11, "hit":1},
            {"c1": 0b11, "m1":0b00, "c2": 0b11, "m2":0b00, "hit":0},
            {"c1": 0b00, "m1":0b11, "c2": 0b00, "m2":0b11, "hit":0},
            {"c1": 0b01, "m1":0b10, "c2": 0b10, "m2":0b00, "hit":0},   
            {"c1": 0b01, "m1":0b10, "c2": 0b10, "m2":0b10, "hit":0},
            {"c1": 0b01, "m1":0b10, "c2": 0b10, "m2":0b01, "hit":1},
            {"c1": 0b01, "m1":0b11, "c2": 0b00, "m2":0b10, "hit":0},
            ]

        for test in tests:
            f1 = p.ShapeFilter(categories = test["c1"], mask=test["m1"])
            f2 = p.ShapeFilter(categories = test["c2"], mask=test["m2"])
            s1.filter = f1
            hit = s.point_query_nearest((0,0), 0, f2)
            self.assertEqual(hit != None, test["hit"], 
                "Got {}!=None, expected {} for test: {}".format(hit, test["hit"], test))

    def testPointQuery(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        b1.position = 19, 0
        s1 = p.Circle(b1, 10)
        s.add(s1)

        b2 = p.Body(1, 1)
        b2.position = 0, 0
        s2 = p.Circle(b2, 10)
        s.add(s2)
        s1.filter = p.ShapeFilter(categories = 0b10, mask=0b01)
        hits = s.point_query((23,0), 0, p.ShapeFilter(categories = 0b01, mask=0b10))

        self.assertEqual(len(hits), 1)
        self.assertEqual(hits[0].shape, s1)
        self.assertEqual(hits[0].point, (29,0))
        self.assertEqual(hits[0].distance, -6)
        self.assertEqual(hits[0].gradient, (1,0))

        hits = s.point_query((30,0), 0, p.ShapeFilter())
        self.assertEqual(len(hits), 0)

        hits = s.point_query((30,0), 30, p.ShapeFilter())
        self.assertEqual(len(hits), 2)
        self.assertEqual(hits[0].shape, s2)
        self.assertEqual(hits[0].point, (10,0))
        self.assertEqual(hits[0].distance, 20)
        self.assertEqual(hits[0].gradient, (1,0))

        self.assertEqual(hits[1].shape, s1)
        self.assertEqual(hits[1].point, (29,0))
        self.assertEqual(hits[1].distance, 1)
        self.assertEqual(hits[1].gradient, (1,0))


    def testPointQuerySensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hits = s.point_query((0,0), 100, p.ShapeFilter())
        self.assertEqual(len(hits), 1)


    def testPointQueryNearest(self):
        s = p.Space()
        b1 = p.Body(1,1)
        b1.position = 19,0
        s1 = p.Circle(b1, 10)
        s.add(s1)

        hit = s.point_query_nearest((23,0), 0, p.ShapeFilter())
        self.assertEqual(hit.shape, s1)
        self.assertEqual(hit.point, (29,0))
        self.assertEqual(hit.distance, -6)
        self.assertEqual(hit.gradient, (1,0))

        hit = s.point_query_nearest((30,0), 0, p.ShapeFilter())
        self.assertEqual(hit, None)

        hit = s.point_query_nearest((30,0), 10, p.ShapeFilter())
        self.assertEqual(hit.shape, s1)
        self.assertEqual(hit.point, (29,0))
        self.assertEqual(hit.distance, 1)
        self.assertEqual(hit.gradient, (1,0))

    def testPointQueryNearestSensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hit = s.point_query_nearest((0,0), 100, p.ShapeFilter())
        self.assertEqual(hit, None)


    def testBBQuery(self):
        s = p.Space()

        b1 = p.Body(1,1)
        b1.position = 19,0
        s1 = p.Circle(b1, 10)
        s.add(s1)

        b2 = p.Body(1, 1)
        b2.position = 0, 0
        s2 = p.Circle(b2, 10)
        s.add(s2)

        bb = p.BB(-7, -7, 7, 7)
        hits = s.bb_query(bb, p.ShapeFilter())
        self.assertEqual(len(hits), 1)
        self.assertTrue(s2 in hits)
        self.assertTrue(s1 not in hits)


    def testBBQuerySensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hits = s.bb_query(p.BB(0,0,10,10), p.ShapeFilter())
        self.assertEqual(len(hits), 1)


    def testShapeQuery(self):
        b = p.Body(body_type=p.Body.KINEMATIC)
        s = p.Circle(b, 2)
        b.position = 20,1

        hits = self.s.shape_query(s)
        
        self.assertEqual(len(hits), 1)
        self.assertEqual(self.s2, hits[0].shape)


    def testShapeQuerySensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hits = s.shape_query(p.Circle(None, 200))
        self.assertEqual(len(hits), 1)


    def testStaticPointQueries(self):
        b = p.Body(body_type=p.Body.KINEMATIC)
        c = p.Circle(b, 10)
        b.position = -50,-50

        self.s.add(c)

        hit = self.s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit.shape, c)

        hits = self.s.point_query( (-50,-55), 0, p.ShapeFilter())
        self.assertEqual(hits[0].shape, c)

    def testReindexShape(self):
        s = p.Space()
        
        b = p.Body(body_type=p.Body.KINEMATIC)
        c = p.Circle(b, 10)

        s.add(c)

        b.position = -50,-50
        hit = self.s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit, None)
        s.reindex_shape(c)
        hit = s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit.shape, c)
        
    def testReindexShapesForBody(self):
        s = p.Space()
        b = p.Body(body_type=p.Body.STATIC)
        c = p.Circle(b, 10)

        s.add(c)

        b.position = -50,-50
        hit = s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit, None)
        s.reindex_shapes_for_body(b)
        
        hit = s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit.shape, c)

    def testReindexStatic(self):
        s = p.Space()
        b = p.Body(body_type=p.Body.STATIC)
        c = p.Circle(b, 10)

        s.add(c)

        b.position = -50,-50
        hit = s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit, None)
        s.reindex_static()
        hit = s.point_query_nearest( (-50,-55), 0, p.ShapeFilter() )
        self.assertEqual(hit.shape, c)

    def testReindexStaticCollision(self):
        s = p.Space()
        b1 = p.Body(10, 1000)
        c1 = p.Circle(b1, 10)
        b1.position = 20, 20

        b2 = p.Body(body_type=p.Body.STATIC)
        s2 = p.Segment(b2, (-10,0), (10,0),1)

        s.add(b1,c1)
        s.add(s2)

        s2.unsafe_set_endpoints((-10,0), (100,0))
        s.gravity = 0, -100

        for x in range(10):
            s.step(.1)
            
        self.assertTrue(b1.position.y < 0)

        b1.position = 20,20
        b1.velocity = 0,0
        s.reindex_static()

        for x in range(10):
            self.s.step(.1)

        self.assertTrue(b1.position.y > 10)

    def testSegmentQuery(self):
        s = p.Space()

        b1 = p.Body(1,1)
        b1.position = 19,0
        s1 = p.Circle(b1, 10)
        s.add(s1)

        b2 = p.Body(1, 1)
        b2.position = 0, 0
        s2 = p.Circle(b2, 10)
        s.add(s2)

        hits = s.segment_query((-13, 0), (131,0), 0, p.ShapeFilter())

        self.assertEqual(len(hits), 2)
        self.assertEqual(hits[0].shape, s2)
        self.assertEqual(hits[0].point, (-10,0))
        self.assertEqual(hits[0].normal, (-1,0))
        self.assertAlmostEqual(hits[0].alpha, 0.0208333333333)

        self.assertEqual(hits[1].shape, s1)
        self.assertEqual(hits[1].point, (9,0))
        self.assertEqual(hits[1].normal, (-1,0))
        self.assertAlmostEqual(hits[1].alpha, 0.1527777777777)

        hits = s.segment_query((-13, 50), (131,50), 0, p.ShapeFilter())
        self.assertEqual(len(hits), 0)

    def testSegmentQuerySensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hits = s.segment_query((-20,0), (20,0), 1, p.ShapeFilter())
        self.assertEqual(len(hits), 1)

    def testSegmentQueryFirst(self):
        s = p.Space()

        b1 = p.Body(1,1)
        b1.position = 19,0
        s1 = p.Circle(b1, 10)
        s.add(s1)

        b2 = p.Body(1, 1)
        b2.position = 0, 0
        s2 = p.Circle(b2, 10)
        s.add(s2)

        hit = s.segment_query_first((-13, 0), (131,0), 0, p.ShapeFilter())

        self.assertEqual(hit.shape, s2)
        self.assertEqual(hit.point, (-10,0))
        self.assertEqual(hit.normal, (-1,0))
        self.assertAlmostEqual(hit.alpha, 0.0208333333333)

        hit = s.segment_query_first((-13, 50), (131,50), 0, p.ShapeFilter())
        self.assertEqual(hit, None)

    def testSegmentQueryFirstSensor(self):
        s = p.Space()
        c = p.Circle(s.static_body, 10)
        c.sensor = True
        s.add(c)
        hit = s.segment_query_first((-20,0), (20,0), 1, p.ShapeFilter())
        self.assertIsNone(hit)

    def testStaticSegmentQueries(self):
        b = p.Body(body_type=p.Body.KINEMATIC)
        c = p.Circle(b, 10)
        b.position = -50,-50

        self.s.add(c)

        hit = self.s.segment_query_first( (-70,-50), (-30, -50), 0, p.ShapeFilter() )
        self.assertEqual(hit.shape, c)
        hits = self.s.segment_query( (-70,-50), (-30, -50), 0, p.ShapeFilter() )
        self.assertEqual(hits[0].shape, c)

    def testCollisionHandlerBegin(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        s.add(b1, c1, b2, c2)
        
        
        self.hits = 0
        def begin(space, arb, data):
            self.hits += h.data["test"]
            return True

        h = s.add_collision_handler(0, 0)
        h.data["test"] = 1
        h.begin = begin

        for x in range(10):
            s.step(0.1)

        self.assertEqual(self.hits, 1)

    def testCollisionHandlerBeginNoReturn(self):        
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        s.add(b1, c1, b2, c2)
        
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            def begin(space, arb, data):
                return

            s.add_collision_handler(0,0).begin = begin
            s.step(0.1)
            
            self.assertEqual(len(w),1)
            self.assertTrue(issubclass(w[-1].category,UserWarning))

    def testCollisionHandlerPreSolve(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        c1.collision_type = 1
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        s.add(b1, c1, b2, c2)

        d = {}
        def pre_solve(arb, space, data):
            d["shapes"] = arb.shapes
            d["space"] = space
            d["test"] = data["test"]
            return True

        h = s.add_collision_handler(0,1)
        h.data["test"] = 1
        h.pre_solve = pre_solve
        s.step(0.1)
        self.assertEqual(c1, d["shapes"][1])
        self.assertEqual(c2, d["shapes"][0])
        self.assertEqual(s, d["space"])
        self.assertEqual(1, d["test"])

    def testCollisionHandlerPreSolveNoReturn(self):        
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        s.add(b1, c1, b2, c2)
        
        def pre_solve(arb, space, data):
            return
        s.add_collision_handler(0,0).pre_solve = pre_solve
        
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
    
            s.step(0.1)

            self.assertEqual(len(w),1)
            self.assertTrue(issubclass(w[-1].category,UserWarning))

    def testCollisionHandlerPostSolve(self):
        self.hit = 0
        def post_solve(arb, space, data):
            self.hit += 1

        self.s.add_collision_handler(0,0).post_solve = post_solve
        self.s.step(0.1)
        self.assertEqual(self.hit, 1)

    def testCollisionHandlerSeparate(self):
        s = p.Space()

        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b1.position = 9,11

        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        b2.position = 0,0

        s.add(b1, c1, b2, c2)
        s.gravity = 0,-100

        self.separated = False
        def separate(arb, space, data):
            self.separated = data["test"]

        h = s.add_collision_handler(0,0)
        h.data["test"] = True
        h.separate = separate

        for x in range(10):
            s.step(0.1)

        self.assertTrue(self.separated)

    def testWildcardCollisionHandler(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        s.add(b1, c1, b2, c2)
        
        d = {}
        def pre_solve(arb, space, data):
            d["shapes"] = arb.shapes
            d["space"] = space
            return True

        s.add_wildcard_collision_handler(1).pre_solve = pre_solve
        s.step(0.1)
        
        self.assertEqual({}, d)
        
        c1.collision_type = 1
        s.step(0.1)

        self.assertEqual(c1, d["shapes"][0])
        self.assertEqual(c2, d["shapes"][1])
        self.assertEqual(s, d["space"])

    def testDefaultCollisionHandler(self):
        s = p.Space()
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        c1.collision_type = 1
        b2 = p.Body(1, 1)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        s.add(b1, c1, b2, c2)
        
        d = {}
        def pre_solve(arb, space, data):
            d["shapes"] = arb.shapes
            d["space"] = space
            return True

        s.add_default_collision_handler().pre_solve = pre_solve
        s.step(0.1)
        
        self.assertEqual(c1, d["shapes"][1])
        self.assertEqual(c2, d["shapes"][0])
        self.assertEqual(s, d["space"])
        

    def testPostStepCallback(self):
        s = p.Space()
        b1, b2 = p.Body(1, 3), p.Body(10, 100)
        s.add(b1, b2)
        b1.position = 10, 0
        b2.position = 20, 0
        s1, s2 = p.Circle(b1, 5), p.Circle(b2, 10)
        s.add(s1, s2)

        self.calls = 0
        def callback(space, key, shapes, test_self):
            for shape in shapes:
                s.remove(shape)
            test_self.calls += 1
        
        def pre_solve(arb, space, data):
            # note that we dont pass on the whole arbiters object, instead
            # we take only the shapes.
            space.add_post_step_callback(callback, 0, arb.shapes, test_self = self)
            return True

        ch = s.add_collision_handler(0,0).pre_solve = pre_solve
        
        s.step(0.1)
        self.assertEqual([], s.shapes)
        self.assertEqual(self.calls, 1)
        
        s.step(0.1)

        self.assertEqual(self.calls, 1)

    def testDebugDraw(self):
        s = p.Space()

        b1 = p.Body(1,3)
        s1 = p.Circle(b1, 5)
        s.add(b1, s1)
        s.step(1)
        o = p.SpaceDebugDrawOptions()

        if sys.version_info >= (3, 0):
            new_out = io.StringIO()
        else:
            new_out = io.BytesIO()
        sys.stdout = new_out
        try:
            s.debug_draw(o)
        finally:
            sys.stdout = sys.__stdout__

        if sys.version_info >= (3, 0):
            msg = ("draw_circle (Vec2d(0.0, 0.0), 0.0, 5.0, "
            "SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), "
            "SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0))\n")
        else:
            msg = ("('draw_circle', (Vec2d(0.0, 0.0), 0.0, 5.0, "
            "SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), "
            "SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0)))\n")
        self.assertEqual(msg, new_out.getvalue())

    def testCopyMethods(self):
        self._testCopyMethod(lambda x: pickle.loads(pickle.dumps(x)))
        self._testCopyMethod(lambda x: copy.deepcopy(x))
        self._testCopyMethod(lambda x: x.copy())

    def _testCopyMethod(self, copy_func):
        s = p.Space(threaded=True)
        s.iterations = 2
        s.gravity = 3,4
        s.damping = 5
        s.idle_speed_threshold = 6
        s.sleep_time_threshold = 7
        s.collision_slop = 8
        s.collision_bias = 9
        s.collision_persistence = 10
        s.threads = 2

        b1 = p.Body(1,2)
        b2 = p.Body(3,4)
        c1 = p.Circle(b1, 7)
        c2 = p.Circle(b1, 8)
        c3 = p.Circle(s.static_body, 9)
        s.add(b1,b2, c1, c2, c3)

        s.static_body.custom = "x"

        j1 = p.PinJoint(b1, b2)
        j2 = p.PinJoint(s.static_body, b2)
        s.add(j1,j2)

        h = s.add_default_collision_handler()
        h.begin = f1

        h = s.add_wildcard_collision_handler(1)
        h.pre_solve = f1

        h = s.add_collision_handler(1, 2)
        h.post_solve = f1

        h = s.add_collision_handler(3,4)
        h.separate = f1

        s2 = copy_func(s)
        
        self.assertEqual(s.threaded, s2.threaded)
        self.assertEqual(s.iterations, s2.iterations)
        self.assertEqual(s.gravity, s2.gravity)
        self.assertEqual(s.damping, s2.damping)
        self.assertEqual(s.idle_speed_threshold, s2.idle_speed_threshold)
        self.assertEqual(s.sleep_time_threshold, s2.sleep_time_threshold)
        self.assertEqual(s.collision_slop, s2.collision_slop)
        self.assertEqual(s.collision_bias, s2.collision_bias)
        self.assertEqual(s.collision_persistence, s2.collision_persistence)
        self.assertEqual(s.threads, s2.threads)

        self.assertEqual(sorted([c.radius for c in s2.shapes]), [7,8,9])
        self.assertEqual(sorted([b.mass for b in s2.bodies]), [1,3])
        self.assertEqual(s.static_body.custom, s2.static_body.custom)
        ja = [j.a for j in s2.constraints]
        self.assertIn(s2.static_body, ja)

        h2 = s2.add_default_collision_handler()
        self.assertIsNotNone(h2.begin)
        self.assertIsNone(h2.pre_solve)
        self.assertIsNone(h2.post_solve)
        self.assertIsNone(h2.separate)

        h2 = s2.add_wildcard_collision_handler(1)
        self.assertIsNone(h2.begin)
        self.assertIsNotNone(h2.pre_solve)
        self.assertIsNone(h2.post_solve)
        self.assertIsNone(h2.separate)

        h2 = s2.add_collision_handler(1,2)
        self.assertIsNone(h2.begin)
        self.assertIsNone(h2.pre_solve)
        self.assertIsNotNone(h2.post_solve)
        self.assertIsNone(h2.separate)

        h2 = s2.add_collision_handler(3,4)
        self.assertIsNone(h2.begin)
        self.assertIsNone(h2.pre_solve)
        self.assertIsNone(h2.post_solve)
        self.assertIsNotNone(h2.separate)

def f1(*args, **kwargs):
    pass

####################################################################
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()



"""This module contain functions for automatic generation of geometry, for 
example from an image.

Example::

    >>> import pymunk
    >>> from pymunk.autogeometry import march_soft
    >>> img = [
    ...     "  xx   ",
    ...     "  xx   ",
    ...     "  xx   ",
    ...     "  xx   ",
    ...     "  xx   ",
    ...     "  xxxxx",
    ...     "  xxxxx",
    ... ]
    >>> segments = []

    >>> def segment_func(v0, v1):
    ...     segments.append((tuple(v0), tuple(v1)))
    >>> def sample_func(point):
    ...     x = int(point.x)
    ...     y = int(point.y)
    ...     return 1 if img[y][x] == "x" else 0

    >>> march_soft(pymunk.BB(0,0,6,6), 7, 7, .5, segment_func, sample_func)
    >>> print(len(segments))
    13

The information in segments can now be used to create geometry, for example as 
a Pymunk Poly or Segment::

    >>> s = pymunk.Space()
    >>> for (a,b) in segments:
    ...     segment = pymunk.Segment(s.static_body, a, b, 5)  
    ...     s.add(segment)


"""
__docformat__ = "reStructuredText"

__all__ = ["is_closed", "simplify_curves", "simplify_vertexes", 
    "to_convex_hull", "convex_decomposition", "PolylineSet", "march_soft", 
    "march_hard"]

import collections

from ._chipmunk_cffi import lib, ffi
from .vec2d import Vec2d
from .bb import BB

def _to_chipmunk(polyline):
    l = len(polyline)
    _line = ffi.new("cpPolyline *", {"verts": l})
    _line.count = l
    _line.capacity = l
    _line.verts = list(map(tuple, polyline))
    return _line

def _from_polyline_set(_set):
    lines = []
    for i in range(_set.count):
        line = []
        for j in range(_set.lines[i].count):
            line.append(Vec2d._fromcffi(_set.lines[i].verts[j]))
        lines.append(line)
    return lines
    
def is_closed(polyline):
    """Returns true if the first vertex is equal to the last.
    
    :param polyline: Polyline to simplify.
    :type polyline: [(float,float)]
    :rtype: `bool`
    """
    return bool(lib.cpPolylineIsClosed(_to_chipmunk(polyline)))

def simplify_curves(polyline, tolerance):
    """Returns a copy of a polyline simplified by using the Douglas-Peucker 
    algorithm.

    This works very well on smooth or gently curved shapes, but not well on 
    straight edged or angular shapes.

    :param polyline: Polyline to simplify.
    :type polyline: [(float,float)]
    :param float tolerance: A higher value means more error is tolerated.
    :rtype: [(float,float)]
    """

    _line = lib.cpPolylineSimplifyCurves(_to_chipmunk(polyline), tolerance)
    simplified = []
    for i in range(_line.count):
        simplified.append(Vec2d._fromcffi(_line.verts[i]))
    return simplified

def simplify_vertexes(polyline, tolerance):
    """Returns a copy of a polyline simplified by discarding "flat" vertexes.
        
    This works well on straight edged or angular shapes, not as well on smooth 
    shapes.    

    :param polyline: Polyline to simplify.
    :type polyline: [(float,float)]
    :param float tolerance: A higher value means more error is tolerated.
    :rtype: [(float,float)]
    """
    _line = lib.cpPolylineSimplifyVertexes(_to_chipmunk(polyline), tolerance)
    simplified = []
    for i in range(_line.count):
        simplified.append(Vec2d._fromcffi(_line.verts[i]))
    return simplified

def to_convex_hull(polyline, tolerance):
    """Get the convex hull of a polyline as a looped polyline.

    :param polyline: Polyline to simplify.
    :type polyline: [(float,float)]
    :param float tolerance: A higher value means more error is tolerated.
    :rtype: [(float,float)]
    """
    _line = lib.cpPolylineToConvexHull(_to_chipmunk(polyline), tolerance)
    hull = []
    for i in range(_line.count):
        hull.append(Vec2d._fromcffi(_line.verts[i]))
    return hull

def convex_decomposition(polyline, tolerance):
    """Get an approximate convex decomposition from a polyline.

    Returns a list of convex hulls that match the original shape to within 
    tolerance.
    
    .. note:: 
        If the input is a self intersecting polygon, the output might end up 
        overly simplified.

    :param polyline: Polyline to simplify.
    :type polyline: [(float,float)]
    :param float tolerance: A higher value means more error is tolerated.
    :rtype: [(float,float)]    
    """
    _line = _to_chipmunk(polyline)
    _set = lib.cpPolylineConvexDecomposition(_line, tolerance)
    return _from_polyline_set(_set)


class PolylineSet(collections.Sequence):
    """A set of Polylines. 
    
    Mainly intended to be used for its :py:meth:`collect_segment` function 
    when generating geometry with the :py:func:`march_soft` and 
    :py:func:`march_hard` functions.    
    """
    def __init__(self):
        def free(_set):
            lib.cpPolylineSetFree(_set, True)
        self._set = ffi.gc(lib.cpPolylineSetNew(), free)

    def collect_segment(self, v0, v1):
        """Add a line segment to a polyline set.
        
        A segment will either start a new polyline, join two others, or add to 
        or loop an existing polyline. This is mostly intended to be used as a 
        callback directly from :py:func:`march_soft` or :py:func:`march_hard`.
        
        :param v0: Start of segment
        :type v0: (float,float)
        :param v1: End of segment
        :type v1: (float,float)
        """
        lib.cpPolylineSetCollectSegment(tuple(v0), tuple(v1), self._set)

    def __len__(self):
        return self._set.count

    def __getitem__(self, key):
        if key >= self._set.count:
            raise IndexError
        line = []
        for i in range(self._set.lines[key].count):
            line.append(Vec2d._fromcffi(self._set.lines[key].verts[i]))
        return line

def march_soft(bb, x_samples, y_samples, threshold, segment_func, sample_func):
    """Trace an *anti-aliased* contour of an image along a particular threshold.

    The given number of samples will be taken and spread across the bounding 
    box area using the sampling function and context. 

    :param BB bb: Bounding box of the area to sample within
    :param int x_samples: Number of samples in x
    :param int y_samples: Number of samples in y
    :param float threshold: A higher value means more error is tolerated
    :param segment_func: The segment function will be called for each segment 
        detected that lies along the density contour for threshold. 
    :type segment_func: ``func(v0 : Vec2d, v1 : Vec2d)``
    :param sample_func: The sample function will be called for 
        x_samples * y_samples spread across the bounding box area, and should 
        return a float. 
    :type sample_func: ``func(point: Vec2d) -> float``
    """
    
    @ffi.callback("cpMarchSegmentFunc")
    def _seg_f(v0, v1, _data):
        segment_func(Vec2d._fromcffi(v0), Vec2d._fromcffi(v1))
        
    @ffi.callback("cpMarchSampleFunc")
    def _sam_f(point, _data):
        return sample_func(Vec2d._fromcffi(point))
    
    lib.cpMarchSoft(bb._bb, x_samples, y_samples, threshold, 
        _seg_f, ffi.NULL, _sam_f, ffi.NULL)

def march_hard(bb, x_samples, y_samples, threshold, segment_func, sample_func):
    """Trace an *aliased* curve of an image along a particular threshold.

    The given number of samples will be taken and spread across the bounding 
    box area using the sampling function and context. 

    :param BB bb: Bounding box of the area to sample within
    :param int x_samples: Number of samples in x
    :param int y_samples: Number of samples in y
    :param float threshold: A higher value means more error is tolerated
    :param segment_func: The segment function will be called for each segment 
        detected that lies along the density contour for threshold. 
    :type segment_func: ``func(v0 : Vec2d, v1 : Vec2d)``
    :param sample_func: The sample function will be called for 
        x_samples * y_samples spread across the bounding box area, and should 
        return a float. 
    :type sample_func: ``func(point: Vec2d) -> float``
    """
    
    @ffi.callback("cpMarchSegmentFunc")
    def _seg_f(v0, v1, _data):
        segment_func(Vec2d._fromcffi(v0), Vec2d._fromcffi(v1))
        
    @ffi.callback("cpMarchSampleFunc")
    def _sam_f(point, _data):
        return sample_func(Vec2d._fromcffi(point))
    
    lib.cpMarchHard(bb._bb, x_samples, y_samples, threshold, 
        _seg_f, ffi.NULL, _sam_f, ffi.NULL)



"""This module contain functions used to load the chipmunk dll/lib file"""

import os.path
import platform
import sys, imp, os

def load_library(ffi, libname, debug_lib=True):
    # lib gets loaded from
    # 32bit python: pymunk/libchipmunk.so, libchipmunk.dylib or chipmunk.dll
    # 64 bit python pymunk/libchipmunk64.so, libchipmunk.dylib or chipmunk64.dll

    s = platform.system()
    if sys.maxsize > 2**32:
        arch = "64"
    else:
        arch = "32"

    # we use *nix library naming as default
    pattern = "lib%s.so"
    if s in ('Windows', 'Microsoft'):
        pattern = "%s.dll"
    elif s == 'Darwin':
        pattern = "lib%s.dylib"

    path = os.path.dirname(os.path.abspath(__file__))
    libfn = pattern % libname
    lib_path = os.path.join(path, libfn)

    # A frozen app may not:
    #   - have a site-packages.zip
    #   - or include the data inside the executable.
    # It may have a .so/.dll.dylib in a normal folder.
    try:
        if not os.path.exists(lib_path) and \
            hasattr(sys, "frozen") or \
            hasattr(sys, "importers") or \
            hasattr(imp, "is_frozen") and imp.is_frozen("__main__"):

            if 'site-packages.zip' in __file__:
                path = os.path.join(os.path.dirname(os.getcwd()), 'Frameworks')
            else:
                path = os.path.dirname(os.path.abspath(sys.executable))
            lib_path = os.path.join(path, libfn)
    except:
        pass

    if debug_lib:
        print ("Loading chipmunk for %s (%sbit) [%s]" % (s, arch, lib_path))
    try:
        lib = ffi.dlopen(lib_path)
    except OSError:
        print ("""
Failed to load Pymunk library.

This error usually means that you don't have a compiled version of Chipmunk in
the correct spot where Pymunk can find it. If you tried to run Pymunk without
installing it properly this can be the result.

The good news is that it is usually enough (at least on *nix and OS X) to
run the build command:

You compile Chipmunk with
> python setup.py build_ext --inplace
and then verify with
> python -m pymunk.test

(for complete instructions please see the readme file)

Another cause of this problem could be if you didnt included the Chipmunk
library when using a freeze tool such as Py2exe or PyInstaller. Please see the
examples for how to include the library file when freezing a binary.

If it still doesnt work, please report as a bug on the issue tracker at
https://github.com/viblo/pymunk/issues
Remember to include information about your OS, which version of python you use
and the version of pymunk you tried to run. A description of what you did to
trigger the error is also good. Please include the exception traceback if any
(usually found below this message).
""")
        raise
    return lib, lib_path



print("__init__", __name__, __file__)

def init():
    print("init")

if __name__ == '__main__':
    import os
    import sys
    #sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
    init()


import math
import pickle
import unittest

import pymunk as p
from pymunk.vec2d import Vec2d


class UnitTestVec2d(unittest.TestCase):
    
    def testCreationAndAccess(self):
        v = Vec2d()
        self.assertEqual(v.x, 0)
        self.assertEqual(v[0], 0)
        self.assertEqual(v.y, 0)
        self.assertEqual(v[1], 0)
    
        v = Vec2d(3,5)
        self.assertEqual(v.x, 3)
        self.assertEqual(v[0], 3)
        self.assertEqual(v.y, 5)
        self.assertEqual(v[1], 5)
    
        v = Vec2d(111, 222)
        self.assertTrue(v.x == 111 and v.y == 222)
        v.x = 333
        v[1] = 444
        self.assertTrue(v[0] == 333 and v[1] == 444)
        
        v = Vec2d(3,5)
        self.assertEqual(len(v), 2)
        self.assertEqual(list(v), [3,5])
        self.assertEqual(tuple(v), (3,5))

    def testMath(self):
        v = Vec2d(111,222)
        self.assertEqual(v + 1, Vec2d(112, 223))
        self.assertTrue(v - 2 == [109, 220])
        self.assertTrue(v * 3 == (333, 666))
        self.assertTrue(v / 2.0 == Vec2d(55.5, 111))
        #self.assertTrue(v / 2 == (55, 111)) # Not supported since this is a c_float structure in the bottom
        self.assertTrue(v ** Vec2d(2, 3) == [12321, 10941048])
        self.assertTrue(v + [-11, 78] == Vec2d(100, 300))
        #self.assertTrue(v / [11,2] == [10,111]) # Not supported since this is a c_float structure in the bottom

    def testReverseMath(self):
        v = Vec2d(111, 222)
        self.assertTrue(1 + v == Vec2d(112, 223))
        self.assertTrue(2 - v == [-109, -220])
        self.assertTrue(3 * v == (333, 666))
        #self.assertTrue([222,999] / v == [2,4]) # Not supported since this is a c_float structure in the bottom
        self.assertTrue([111, 222] ** Vec2d(2, 3) == [12321, 10941048])
        self.assertTrue([-11, 78] + v == Vec2d(100, 300))

    def testUnary(self):
        v = Vec2d(111, 222)
        v = -v
        self.assertTrue(v == [-111, -222])
        v = abs(v)
        self.assertTrue(v == [111, 222])

    def testLength(self):
        v = Vec2d(3,4)
        self.assertTrue(v.length == 5)
        self.assertTrue(v.get_length_sqrd() == 25)
        self.assertTrue(v.normalize_return_length() == 5)
        self.assertAlmostEqual(v.length, 1)
        v.length = 5
        self.assertTrue(v == Vec2d(3, 4))
        v2 = Vec2d(10, -2)
        self.assertTrue(v.get_distance(v2) == (v - v2).get_length())
        
    def testAnglesDegrees(self):            
        v = Vec2d(0, 3)
        self.assertEqual(v.angle_degrees, 90)
        v2 = Vec2d(v)
        v.rotate_degrees(-90)
        self.assertEqual(v.get_angle_degrees_between(v2), 90)
        v2.angle_degrees -= 90
        self.assertEqual(v.length, v2.length)
        self.assertEqual(v2.angle_degrees, 0)
        self.assertEqual(v2, [3, 0])
        self.assertTrue((v - v2).length < .00001)
        self.assertEqual(v.length, v2.length)
        v2.rotate_degrees(300)
        self.assertAlmostEqual(v.get_angle_degrees_between(v2), -60) # Allow a little more error than usual (floats..)
        v2.rotate_degrees(v2.get_angle_degrees_between(v))
        angle = v.get_angle_degrees_between(v2)
        self.assertAlmostEqual(v.get_angle_degrees_between(v2), 0)  

    def testAnglesRadians(self):            
        v = Vec2d(0, 3)
        self.assertEqual(v.angle, math.pi/2.)
        v2 = Vec2d(v)
        v.rotate(-math.pi/2.)
        self.assertEqual(v.get_angle_between(v2), math.pi/2.)
        v2.angle -= math.pi/2.
        self.assertEqual(v.length, v2.length)
        self.assertEqual(v2.angle, 0)
        self.assertEqual(v2, [3, 0])
        self.assertTrue((v - v2).length < .00001)
        self.assertEqual(v.length, v2.length)
        v2.rotate(math.pi/3.*5.)
        self.assertAlmostEqual(v.get_angle_between(v2), -math.pi/3.) # Allow a little more error than usual (floats..)
        v2.rotate(v2.get_angle_between(v))
        angle = v.get_angle_between(v2)
        self.assertAlmostEqual(v.get_angle_between(v2), 0) 

    def testHighLevel(self):
        basis0 = Vec2d(5.0, 0)
        basis1 = Vec2d(0, .5)
        v = Vec2d(10, 1)
        self.assertTrue(v.convert_to_basis(basis0, basis1) == [2, 2])
        self.assertTrue(v.projection(basis0) == (10, 0))
        self.assertTrue(basis0.dot(basis1) == 0)
        
    def testCross(self):
        lhs = Vec2d(1, .5)
        rhs = Vec2d(4, 6)
        self.assertTrue(lhs.cross(rhs) == 4)
        
    def testComparison(self):
        int_vec = Vec2d(3, -2)
        flt_vec = Vec2d(3.0, -2.0)
        zero_vec = Vec2d(0, 0)
        self.assertTrue(int_vec == flt_vec)
        self.assertTrue(int_vec != zero_vec)
        self.assertTrue((flt_vec == zero_vec) == False)
        self.assertTrue((flt_vec != int_vec) == False)
        self.assertTrue(int_vec == (3, -2))
        self.assertTrue(int_vec != [0, 0])
        self.assertTrue(int_vec != 5)
        self.assertTrue(int_vec != [3, -2, -5])
    
    def testInplace(self):
        inplace_vec = Vec2d(5, 13)
        inplace_ref = inplace_vec
        inplace_src = Vec2d(inplace_vec)    
        inplace_vec *= .5
        inplace_vec += .5
        inplace_vec /= (3, 6)
        inplace_vec += Vec2d(-1, -1)
        alternate = (inplace_src*.5 + .5)/Vec2d(3, 6) + [-1, -1]
        self.assertEqual(inplace_vec, inplace_ref)
        self.assertEqual(inplace_vec, alternate)
    
    def testPickle(self):
        testvec = Vec2d(5, .3)
        testvec_str = pickle.dumps(testvec)
        loaded_vec = pickle.loads(testvec_str)
        self.assertEqual(testvec, loaded_vec)

if __name__ == "__main__":
    print ("testing pymunk.vec2d version " + p.version)
    unittest.main()


# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2017 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""
Pymunk is a easy-to-use pythonic 2d physics library that can be used whenever
you need 2d rigid body physics from Python.

Homepage: http://www.pymunk.org

This is the main containing module of Pymunk. It contains among other things
the very central Space, Body and Shape classes.

When you import this module it will automatically load the chipmunk library
file. As long as you haven't turned off the debug mode a print will show
exactly which Chipmunk library file it loaded. For example::

    >>> import pymunk

    Loading chipmunk for Windows (32bit) [C:\code\pymunk\chipmunk.dll]
"""

__docformat__ = "reStructuredText"


__all__ = ["inf", "version", "chipmunk_version"
        , "Space", "Body", "Shape", "Circle", "Poly", "Segment"
        , "moment_for_circle", "moment_for_poly", "moment_for_segment"
        , "moment_for_box"
        , "SegmentQueryInfo", "ContactPoint", "ContactPointSet", "Arbiter"
        , "CollisionHandler" 
        , "BB", "ShapeFilter"
        , "Transform", "PointQueryInfo", "ShapeQueryInfo"
        , "SpaceDebugDrawOptions"]

import warnings
import sys

from . import _chipmunk_cffi
cp = _chipmunk_cffi.lib
ffi = _chipmunk_cffi.ffi

from .vec2d import Vec2d
from .shape_filter import ShapeFilter
from .transform import Transform
from .contact_point_set import ContactPoint, ContactPointSet
from .arbiter import Arbiter
from .bb import BB
from .body import Body
from .collision_handler import CollisionHandler
from .query_info import PointQueryInfo, SegmentQueryInfo, ShapeQueryInfo
from .shapes import Shape, Circle, Poly, Segment
from .space import Space
from .space_debug_draw_options import SpaceDebugDrawOptions
from pymunk.constraint import *

version = "5.6.0"
"""The release version of this pymunk installation.
Valid only if pymunk was installed from a source or binary
distribution (i.e. not in a checked-out copy from git).
"""

chipmunk_version = "%sR%s" % (ffi.string(cp.cpVersionString), 
    'aef346fb8bac3757c3c6faa019bbf97bafc296d1')
"""The Chipmunk version compatible with this pymunk version.
Other (newer) Chipmunk versions might also work if the new version does not
contain any breaking API changes.

This property does not show a valid value in the compiled documentation, only
when you actually import pymunk and do pymunk.chipmunk_version

The string is in the following format:
<cpVersionString>R<github commit of chipmunk>
where cpVersionString is a version string set by Chipmunk and the git commit
hash corresponds to the git hash of the chipmunk source from
github.com/viblo/Chipmunk2D included with Pymunk. If the Chipmunk version
is a release then the second part will be empty

.. note::
    This is also the version of the Chipmunk source files included in the
    chipmunk_src folder (normally included in the Pymunk source distribution).
"""

chipmunk_path = _chipmunk_cffi.lib_path
"""The path to the Chipmunk library loaded.

Useful in case you are packaging a Pymunk program with for example Py2exe or 
PyInstaller and need to know what library file to include. Please see the 
Py2exe examples in the examples folder of Pymunk for example of this.
"""

inf = float('inf')
"""Infinity that can be passed as mass or inertia to a :py:class:`Body`.

Useful when you for example want a body that cannot rotate, just set its
moment to inf. Just remember that if two objects with both infinite masses
collides the world might explode. Similar effects can happen with infinite
moment.

.. note::
    In previous versions of Pymunk you used inf to create static bodies. This
    has changed. See :py:class:`Body` for details.
"""

def moment_for_circle(mass, inner_radius, outer_radius, offset=(0, 0)):
    """Calculate the moment of inertia for a hollow circle

    inner_radius and outer_radius are the inner and outer diameters.
    (A solid circle has an inner diameter of 0)
    """
    return cp.cpMomentForCircle(mass, inner_radius, outer_radius, tuple(offset))

def moment_for_segment(mass, a, b, radius):
    """ Calculate the moment of inertia for a line segment

    The endpoints a and b are relative to the body
    """
    return cp.cpMomentForSegment(mass, tuple(a), tuple(b), radius)

def moment_for_box(mass, size):
    """Calculate the moment of inertia for a solid box centered on the body.
    
    size should be a tuple of (width, height)
    """
    return cp.cpMomentForBox(mass, size[0], size[1])

def moment_for_poly(mass, vertices,  offset=(0, 0), radius=0):
    """Calculate the moment of inertia for a solid polygon shape.

    Assumes the polygon center of gravity is at its centroid. The offset is
    added to each vertex.
    """
    vs = list(map(tuple, vertices))
    return cp.cpMomentForPoly(mass, len(vertices), vs, tuple(offset), radius)
    
def area_for_circle(inner_radius, outer_radius):
    """Area of a hollow circle."""
    return cp.cpAreaForCircle(inner_radius, outer_radius)

def area_for_segment(a, b, radius):
    """Area of a beveled segment.

    (Will always be zero if radius is zero)
    """
    return cp.cpAreaForSegment(tuple(a), tuple(b), radius)

def area_for_poly(vertices, radius=0):
    """Signed area of a polygon shape.

    Returns a negative number for polygons with a clockwise winding.
    """
    vs = list(map(tuple, vertices))
    return cp.cpAreaForPoly(len(vertices), vs, radius)


#del cp, ct, u



# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2019 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""This submodule contains helper functions to help with quick prototyping 
using pymunk together with kivy.

Intended to help with debugging and prototyping, not for actual production use
in a full application.
"""

__docformat__ = "reStructuredText"

__all__ = ["DrawOptions"]


import math

import kivy
from kivy.graphics import Ellipse, Line, Color, Triangle, Quad, Rectangle

import pymunk
from pymunk.vec2d import Vec2d

class DrawOptions(pymunk.SpaceDebugDrawOptions):
    def __init__(self, **kwargs):
        """Draw a pymunk.Space.
        
        Typical usage::
        
        >>> import pymunk
        >>> import pymunk.pygame_util
        >>> s = pymunk.Space()
        >>> options = pymunk.kivy_util.DrawOptions()
        >>> s.debug_draw(options)
        
        You can control the color of a Shape by setting shape.color to the color 
        you want it drawn in.
        
        >>> c = pymunk.Circle(None, 10)
        >>> c.color = (255, 0, 0, 255) # will draw my_shape in red
                
        See kivy_util.demo.py for a full example
        
        :Param:
                kwargs : You can optionally pass in a pyglet.graphics.Batch
                    If a batch is given all drawing will use this batch to draw 
                    on. If no batch is given a a new batch will be used for the
                    drawing. Remember that if you pass in your own batch you 
                    need to call draw on it yourself.
        
        """
        self.new_batch = False
        
        if "batch" not in kwargs:
            self.new_batch = True
        else:
            self.batch = kwargs["batch"]

        super(DrawOptions, self).__init__()


    def __enter__(self):
        if self.new_batch:
            self.batch = pyglet.graphics.Batch()
    def __exit__(self, type, value, traceback):
        if self.new_batch:
            self.batch.draw()

    def draw_shape(self, shape):
        outline_color = options.shape_outline_color
        fill_color = options.color_for_shape(shape)
        if isinstance(shape, pymunk.Circle):
            # todo adjust by the body pos/angle
            pos = shape.body.position
            angle = shape.body.angle
            radius = shape.radius
            options.draw_circle(pos, angle, radius, 
                outline_color, fill_color)
        elif isinstance(shape, pymunk.Segment):
            a = shape.a # todo adjust by the body pos/angle
            b = shape.b # todo adjust by the body pos/angle
            if shape.radius == 0:
                options.draw_segment(a, b, color)
            else:
                radius = shape.radius
                options.draw_fat_segment(a, b, radius, 
                    outline_color, fill_color)
        elif isinstance(shape, pymunk.Poly):
            verts = shape.get_vertices() # todo adjust by the body pos/angle
            radius = shape.radius
            options.draw_polygon(verts, radius, outline_color, fill_color):


    def ellipse_from_circle(self, shape):
        pos = shape.body.position - (shape.radius, shape.radius)
        e = Ellipse(pos=pos, size=[shape.radius*2, shape.radius*2])
        circle_edge = shape.body.position + Vec2d(shape.radius, 0).rotated(shape.body.angle)
        Color(.17,.24,.31)
        l = Line(points = [shape.body.position.x, shape.body.position.y, circle_edge.x, circle_edge.y])
        return e,l

    def points_from_poly(self, shape):
        body = shape.body
        ps = [p.rotated(body.angle) + body.position for p in shape.get_vertices()]
        vs = []
        for p in ps:
            vs += [p.x, p.y]
        return vs



# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2016 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""This submodule contains helper functions to help with quick prototyping 
using pymunk together with pygame.

Intended to help with debugging and prototyping, not for actual production use
in a full application. The methods contained in this module is opinionated 
about your coordinate system and not in any way optimized. 
"""

__docformat__ = "reStructuredText"

__all__ = [
    "DrawOptions", "get_mouse_pos", "to_pygame", "from_pygame", 
    "positive_y_is_up"
    ]

import pygame

import pymunk
from pymunk.vec2d import Vec2d

positive_y_is_up = True
"""Make increasing values of y point upwards.

When True::

        y
        ^
        |   . (2, 2)
        |
    ----+------ > x
        |
        |     . (3, -2)
        |

When False::

        y
        ^
        |   . (2, -2)
        |
    ----+------ > x
        |
        |      . (3, 2)
        |

"""

class DrawOptions(pymunk.SpaceDebugDrawOptions):
    def __init__(self, surface):
        """Draw a pymunk.Space on a pygame.Surface object.
        
        Typical usage::

        >>> import pymunk
        >>> import pymunk.pygame_util
        >>> surface = pygame.Surface((10,10))
        >>> s = pymunk.Space()
        >>> options = pymunk.pygame_util.DrawOptions(surface)
        >>> s.debug_draw(options)

        Since pygame uses a coordinate system where y points down (compared to
        most other cases where a positive y points upwards), we might want to
        make adjustments for that with the :py:data:`positive_y_is_up` variable.

        By default drawing is done with positive y pointing up, but that will 
        make conversion from pygame coordinate to pymunk coordinate necessary. 
        If you do a lot of those (for example, lots of mouse input) it might be 
        more convenient to set it to False::

        >>> positive_y_is_up = False
        >>> # Draw verything the pygame way, (0,0) in the top left corner 
        >>> positive_y_is_up = True
        >>> # Draw everything the pymunk way, (0,0) in the bottom left corner

        You can control the color of a shape by setting shape.color to the color 
        you want it drawn in.
        
        >>> c = pymunk.Circle(None, 10)
        >>> c.color = pygame.color.THECOLORS["pink"]
        
        See pygame_util.demo.py for a full example
        
        :Parameters:
                surface : pygame.Surface
                    Surface that the objects will be drawn on
        """
        self.surface = surface
        super(DrawOptions, self).__init__()

    def draw_circle(self, pos, angle, radius, outline_color, fill_color):
        p = to_pygame(pos, self.surface)
    
        pygame.draw.circle(self.surface, fill_color, p, _rndint(radius), 0)
        
        circle_edge = pos + Vec2d(radius, 0).rotated(angle)
        p2 = to_pygame(circle_edge, self.surface)
        line_r = 2 if radius > 20 else 1
        pygame.draw.lines(self.surface, outline_color, False, [p,p2], line_r)    

    def draw_segment(self, a, b, color):
        p1 = to_pygame(a, self.surface)
        p2 = to_pygame(b, self.surface)

        pygame.draw.aalines(self.surface, color, False, [p1,p2])

    def draw_fat_segment(self, a, b, radius, outline_color, fill_color):
        p1 = to_pygame(a, self.surface)
        p2 = to_pygame(b, self.surface)
        
        r = _rndint(max(1, radius*2))
        pygame.draw.lines(self.surface, fill_color, False, [p1,p2], r)
        if r > 2:
            orthog = [ abs(p2[1]-p1[1]), abs(p2[0]-p1[0]) ]
            if orthog[0] == 0 and orthog[1] == 0:
                return
            scale = radius / (orthog[0]*orthog[0] + orthog[1]*orthog[1])**0.5
            orthog[0]*=scale; orthog[1]*=scale
            points = [
                ( p1[0]-orthog[0], p1[1]-orthog[1] ),
                ( p1[0]+orthog[0], p1[1]+orthog[1] ),
                ( p2[0]+orthog[0], p2[1]+orthog[1] ),
                ( p2[0]-orthog[0], p2[1]-orthog[1] )
            ]
            pygame.draw.polygon(self.surface, fill_color, points)
            pygame.draw.circle(self.surface, fill_color, 
                (_rndint(p1[0]),_rndint(p1[1])), _rndint(radius))
            pygame.draw.circle(self.surface, fill_color, 
                (_rndint(p2[0]),_rndint(p2[1])), _rndint(radius))
        
    def draw_polygon(self, verts, radius, outline_color, fill_color):
        ps = [to_pygame(v, self.surface) for v in verts]
        ps += [ps[0]]

        pygame.draw.polygon(self.surface, fill_color, ps)

        if radius > 0:
            for i in range(len(verts)):
                a = verts[i]
                b = verts[(i+1) % len(verts)]
                self.draw_fat_segment(a, b, radius, outline_color, 
                    outline_color)

    def draw_dot(self, size, pos, color):
        p = to_pygame(pos, self.surface)
        pygame.draw.circle(self.surface, color, p, _rndint(size), 0)

    
def get_mouse_pos(surface):
    """Get position of the mouse pointer in pymunk coordinates."""
    p = pygame.mouse.get_pos()
    return from_pygame(p, surface)

def to_pygame(p, surface):
    """Convenience method to convert pymunk coordinates to pygame surface 
    local coordinates. 
    
    Note that in case positive_y_is_up is False, this function wont actually do
    anything except converting the point to integers.
    """
    if positive_y_is_up:
        return int(p[0]), surface.get_height()-int(p[1])
    else:
        return int(p[0]), int(p[1])
    
def from_pygame(p, surface):
    """Convenience method to convert pygame surface local coordinates to 
    pymunk coordinates    
    """
    return to_pygame(p,surface)

def _rndint(x): 
    return int(round(x))            



__docformat__ = "reStructuredText"

import weakref
from weakref import WeakSet
import platform
import copy

from . import _chipmunk_cffi
cp = _chipmunk_cffi.lib
ffi = _chipmunk_cffi.ffi    
    
from .vec2d import Vec2d
from .body import Body
from .collision_handler import CollisionHandler
from .query_info import PointQueryInfo, SegmentQueryInfo, ShapeQueryInfo
from .shapes import Shape, Circle, Poly, Segment
from .contact_point_set import ContactPointSet
from ._pickle import PickleMixin
from pymunk.constraint import Constraint


class Space(PickleMixin, object):
    """Spaces are the basic unit of simulation. You add rigid bodies, shapes
    and joints to it and then step them all forward together through time.

    A Space can be copied and pickled. Note that any post step callbacks are 
    not copied. Also note that some internal collision cache data is not copied,
    which can make the simulation a bit unstable the first few steps of the 
    fresh copy.

    Custom properties set on the space will also be copied/pickled.

    Any collision handlers will also be copied/pickled. Note that depending on 
    the pickle protocol used there are some restrictions on what functions can 
    be copied/pickled.

    Example::

    >>> import pymunk, pickle
    >>> space = pymunk.Space()
    >>> space2 = space.copy()
    >>> space3 = pickle.loads(pickle.dumps(space))
    """

    _pickle_attrs_init = ['threaded']
    _pickle_attrs_general = ['iterations', 'gravity', 'damping', 
        'idle_speed_threshold', 'sleep_time_threshold', 'collision_slop',
        'collision_bias', 'collision_persistence', 'threads']

    def __init__(self, threaded=False):
        """Create a new instance of the Space. 
        
        If you set threaded=True the step function will run in threaded mode
        which might give a speedup. Note that even when you set threaded=True 
        you still have to set Space.threads=2 to actually use more than one 
        thread.        
        
        Also note that threaded mode is not available on Windows, and setting 
        threaded=True has no effect on that platform.
        """
        
        self.threaded = threaded and platform.system() != 'Windows' 
        if self.threaded:        
            self._space = ffi.gc(cp.cpHastySpaceNew(), cp.cpHastySpaceFree)
        else:
            self._space = ffi.gc(cp.cpSpaceNew(), cp.cpSpaceFree)

        self._handlers = {} # To prevent the gc to collect the callbacks.

        self._post_step_callbacks = {}
        self._removed_shapes = {}

        self._shapes = {}
        self._bodies = set()
        self._static_body = None
        self._constraints = set()
        
        self._in_step = False
        self._add_later = set()
        self._remove_later = set()


    def _get_self(self):
        return self

    def _get_shapes(self):
        """A list of all the shapes added to this space 
        
        (includes both static and non-static)
        """
        return list(self._shapes.values())
    shapes = property(_get_shapes,
        doc=_get_shapes.__doc__)

    def _get_bodies(self):
        return list(self._bodies)
    bodies = property(_get_bodies,
        doc="""A list of the bodies added to this space""")

    def _get_constraints(self):
        return list(self._constraints)
    constraints = property(_get_constraints,
        doc="""A list of the constraints added to this space""")

    def _get_static_body(self):
        """A dedicated static body for the space. 
        
        You don't have to use it, but because its memory is managed 
        automatically with the space its very convenient. 
        """
        if self._static_body == None:
            b = cp.cpSpaceGetStaticBody(self._space)
            self._static_body = Body._init_with_body(b)
            self._static_body._space = self
        return self._static_body
    static_body = property(_get_static_body, doc=_get_static_body.__doc__)

    def _set_iterations(self, value):
        cp.cpSpaceSetIterations(self._space, value)
    def _get_iterations(self):
        return cp.cpSpaceGetIterations(self._space)
    iterations = property(_get_iterations, _set_iterations
        , doc="""Iterations allow you to control the accuracy of the solver.

        Defaults to 10.

        Pymunk uses an iterative solver to figure out the forces between
        objects in the space. What this means is that it builds a big list of
        all of the collisions, joints, and other constraints between the
        bodies and makes several passes over the list considering each one
        individually. The number of passes it makes is the iteration count,
        and each iteration makes the solution more accurate. If you use too
        many iterations, the physics should look nice and solid, but may use
        up too much CPU time. If you use too few iterations, the simulation
        may seem mushy or bouncy when the objects should be solid. Setting
        the number of iterations lets you balance between CPU usage and the
        accuracy of the physics. Pymunk's default of 10 iterations is
        sufficient for most simple games.
        """)


    def _set_gravity(self, gravity_vector):
        cp.cpSpaceSetGravity(self._space, tuple(gravity_vector))
    def _get_gravity(self):
        return Vec2d._fromcffi(cp.cpSpaceGetGravity(self._space))
    gravity = property(_get_gravity, _set_gravity
        , doc="""Global gravity applied to the space.

        Defaults to (0,0). Can be overridden on a per body basis by writing
        custom integration functions.
        """)

    def _set_damping(self, damping):
        cp.cpSpaceSetDamping(self._space, damping)
    def _get_damping(self):
        return cp.cpSpaceGetDamping(self._space)
    damping = property(_get_damping, _set_damping
        , doc="""Amount of simple damping to apply to the space.

        A value of 0.9 means that each body will lose 10% of its velocity per
        second. Defaults to 1. Like gravity, it can be overridden on a per
        body basis.
        """)

    def _set_idle_speed_threshold(self, idle_speed_threshold):
        cp.cpSpaceSetIdleSpeedThreshold(self._space, idle_speed_threshold)
    def _get_idle_speed_threshold(self):
        return cp.cpSpaceGetIdleSpeedThreshold(self._space)
    idle_speed_threshold = property(_get_idle_speed_threshold
        , _set_idle_speed_threshold
        , doc="""Speed threshold for a body to be considered idle.

        The default value of 0 means the space estimates a good threshold
        based on gravity.
        """)

    def _set_sleep_time_threshold(self, sleep_time_threshold):
        cp.cpSpaceSetSleepTimeThreshold(self._space, sleep_time_threshold)
    def _get_sleep_time_threshold(self):
        return cp.cpSpaceGetSleepTimeThreshold(self._space)
    sleep_time_threshold = property(_get_sleep_time_threshold
        , _set_sleep_time_threshold
        , doc="""Time a group of bodies must remain idle in order to fall
        asleep.

        The default value of `inf` disables the sleeping algorithm.
        """)

    def _set_collision_slop(self, collision_slop):
        cp.cpSpaceSetCollisionSlop(self._space, collision_slop)
    def _get_collision_slop(self):
        return cp.cpSpaceGetCollisionSlop(self._space)
    collision_slop = property(_get_collision_slop
        , _set_collision_slop
        , doc="""Amount of overlap between shapes that is allowed.

        To improve stability, set this as high as you can without noticeable
        overlapping. It defaults to 0.1.
        """)

    def _set_collision_bias(self, collision_bias):
        cp.cpSpaceSetCollisionBias(self._space, collision_bias)
    def _get_collision_bias(self):
        return cp.cpSpaceGetCollisionBias(self._space)
    collision_bias = property(_get_collision_bias
        , _set_collision_bias
        , doc="""Determines how fast overlapping shapes are pushed apart.

        Pymunk allows fast moving objects to overlap, then fixes the overlap
        over time. Overlapping objects are unavoidable even if swept
        collisions are supported, and this is an efficient and stable way to
        deal with overlapping objects. The bias value controls what
        percentage of overlap remains unfixed after a second and defaults
        to ~0.2%. Valid values are in the range from 0 to 1, but using 0 is
        not recommended for stability reasons. The default value is
        calculated as cpfpow(1.0f - 0.1f, 60.0f) meaning that pymunk attempts
        to correct 10% of error ever 1/60th of a second.

        ..Note::
            Very very few games will need to change this value.
        """)

    def _set_collision_persistence(self, collision_persistence):
        cp.cpSpaceSetCollisionPersistence(self._space, collision_persistence)
    def _get_collision_persistence(self):
        return cp.cpSpaceGetCollisionPersistence(self._space)
    collision_persistence = property(_get_collision_persistence
        , _set_collision_persistence
        , doc="""The number of frames the space keeps collision solutions
        around for.

        Helps prevent jittering contacts from getting worse. This defaults
        to 3.

        ..Note::
            Very very few games will need to change this value.
        """)

    def _get_current_time_step(self):
        return cp.cpSpaceGetCurrentTimeStep(self._space)
    current_time_step = property(_get_current_time_step,
        doc="""Retrieves the current (if you are in a callback from
        Space.step()) or most recent (outside of a Space.step() call)
        timestep.
        """)

    def add(self, *objs):
        """Add one or many shapes, bodies or joints to the space

        Unlike Chipmunk and earlier versions of pymunk its now allowed to add
        objects even from a callback during the simulation step. However, the
        add will not be performed until the end of the step.
        """

        if self._in_step:
            self._add_later.update(objs)
            return

        for o in objs:
            if isinstance(o, Body):
                self._add_body(o)
            elif isinstance(o, Shape):
                self._add_shape(o)
            elif isinstance(o, Constraint):
                self._add_constraint(o)
            else:
                for oo in o:
                    self.add(oo)

    def remove(self, *objs):
        """Remove one or many shapes, bodies or constraints from the space

        Unlike Chipmunk and earlier versions of Pymunk its now allowed to
        remove objects even from a callback during the simulation step.
        However, the removal will not be performed until the end of the step.

        .. Note::
            When removing objects from the space, make sure you remove any
            other objects that reference it. For instance, when you remove a
            body, remove the joints and shapes attached to it.
        """

        if self._in_step:
            self._remove_later.update(objs)
            return

        for o in objs:
            if isinstance(o, Body):
                self._remove_body(o)
            elif isinstance(o, Shape):
                self._remove_shape(o)
            elif isinstance(o, Constraint):
                self._remove_constraint(o)
            else:
                for oo in o:
                    self.remove(oo)

    def _add_shape(self, shape):
        """Adds a shape to the space"""
        assert shape._get_shapeid() not in self._shapes, "shape already added to space"
        shape._space = weakref.proxy(self)
        self._shapes[shape._get_shapeid()] = shape
        cp.cpSpaceAddShape(self._space, shape._shape)

    def _add_body(self, body):
        """Adds a body to the space"""
        assert body not in self._bodies, "body already added to space"
        body._space = weakref.proxy(self)
        self._bodies.add(body)
        cp.cpSpaceAddBody(self._space, body._body)

    def _add_constraint(self, constraint):
        """Adds a constraint to the space"""
        assert constraint not in self._constraints, "constraint already added to space"
        self._constraints.add(constraint)
        cp.cpSpaceAddConstraint(self._space, constraint._constraint)

    def _remove_shape(self, shape):
        """Removes a shape from the space"""
        self._removed_shapes[shape._get_shapeid()] = shape
        del self._shapes[shape._get_shapeid()]
        cp.cpSpaceRemoveShape(self._space, shape._shape)
    def _remove_body(self, body):
        """Removes a body from the space"""
        body._space = None
        self._bodies.remove(body)
        cp.cpSpaceRemoveBody(self._space, body._body)
    def _remove_constraint(self, constraint):
        """Removes a constraint from the space"""
        self._constraints.remove(constraint)
        cp.cpSpaceRemoveConstraint(self._space, constraint._constraint)

    def reindex_shape(self, shape):
        """Update the collision detection data for a specific shape in the
        space.
        """
        cp.cpSpaceReindexShape(self._space, shape._shape)

    def reindex_shapes_for_body(self, body):
        """Reindex all the shapes for a certain body."""
        cp.cpSpaceReindexShapesForBody(self._space, body._body)

    def reindex_static(self):
        """Update the collision detection info for the static shapes in the
        space. You only need to call this if you move one of the static shapes.
        """
        cp.cpSpaceReindexStatic(self._space)

    def _get_threads(self):
        if self.threaded:
            return int(cp.cpHastySpaceGetThreads(self._space))
        return 1 

    def _set_threads(self, n):
        if self.threaded:
            cp.cpHastySpaceSetThreads(self._space, n)
    threads=property(_get_threads, _set_threads, 
        doc="""The number of threads to use for running the step function. 
        
        Only valid when the Space was created with threaded=True. Currently the 
        max limit is 2, setting a higher value wont have any effect. The 
        default is 1 regardless if the Space was created with threaded=True, 
        to keep determinism in the simulation. Note that Windows does not 
        support the threaded solver.
        """)


    def step(self, dt):
        """Update the space for the given time step. 
        
        Using a fixed time step is highly recommended. Doing so will increase 
        the efficiency of the contact persistence, requiring an order of 
        magnitude fewer iterations to resolve the collisions in the usual case.

        It is not the same to call step 10 times with a dt of 0.1 and 
        calling it 100 times with a dt of 0.01 even if the end result is 
        that the simulation moved forward 100 units. Performing  multiple 
        calls with a smaller dt creates a more stable and accurate 
        simulation. Therefor it sometimes make sense to have a little for loop
        around the step call, like in this example:

        >>> import pymunk
        >>> s = pymunk.Space()
        >>> steps = 10
        >>> for x in range(steps): # move simulation forward 0.1 seconds:
        ...     s.step(0.1 / steps)
        
        :param float dt: Time step length
        """

        self._in_step = True
        if self.threaded:
            cp.cpHastySpaceStep(self._space, dt)
        else:
            cp.cpSpaceStep(self._space, dt)
        self._removed_shapes = {}
        self._in_step = False

        for objs in self._add_later:
            self.add(objs)
        self._add_later.clear()

        for objs in self._remove_later:
            self.remove(objs)
        self._remove_later.clear()
        
        for key in self._post_step_callbacks:
            self._post_step_callbacks[key](self)  
        
        self._post_step_callbacks = {}  


    def add_collision_handler(self, collision_type_a, collision_type_b):
        """Return the :py:class:`CollisionHandler` for collisions between 
        objects of type collision_type_a and collision_type_b.

        Fill the desired collision callback functions, for details see the
        :py:class:`CollisionHandler` object.

        Whenever shapes with collision types (:py:attr:`Shape.collision_type`) 
        a and b collide, this handler will be used to process the collision 
        events. When a new collision handler is created, the callbacks will all be
        set to builtin callbacks that perform the default behavior (call the
        wildcard handlers, and accept all collisions).

        :param int collision_type_a: Collision type a
        :param int collision_type_b: Collision type b

        :rtype: :py:class:`CollisionHandler`
        """
        
        key = (collision_type_a, collision_type_b) 
        if key in self._handlers:
            return self._handlers[key]
        
        h = cp.cpSpaceAddCollisionHandler(self._space, collision_type_a, collision_type_b)
        ch = CollisionHandler(h, self)
        self._handlers[key] = ch
        return ch
        
    def add_wildcard_collision_handler(self, collision_type_a):
        """Add a wildcard collision handler for given collision type.

        This handler will be used any time an object with this type collides
        with another object, regardless of its type. A good example is a
        projectile that should be destroyed the first time it hits anything.
        There may be a specific collision handler and two wildcard handlers.
        It's up to the specific handler to decide if and when to call the
        wildcard handlers and what to do with their return values.

        When a new wildcard handler is created, the callbacks will all be
        set to builtin callbacks that perform the default behavior. (accept
        all collisions in :py:func:`~CollisionHandler.begin` and 
        :py:func:`~CollisionHandler.pre_solve`, or do nothing for
        :py:func:`~CollisionHandler.post_solve` and 
        :py:func:`~CollisionHandler.separate`.

        :param int collision_type_a: Collision type
        :rtype: :py:class:`CollisionHandler`
        """

        if collision_type_a in self._handlers:
            return self._handlers[collision_type_a]
            
        h = cp.cpSpaceAddWildcardHandler(self._space, collision_type_a)
        ch = CollisionHandler(h, self)
        self._handlers[collision_type_a] = ch
        return ch

    def add_default_collision_handler(self):
        """Return a reference to the default collision handler or that is
        used to process all collisions that don't have a more specific
        handler.

        The default behavior for each of the callbacks is to call
        the wildcard handlers, ANDing their return values together if
        applicable.
        """
        if None in self._handlers:
            return self._handlers[None]
        
        _h = cp.cpSpaceAddDefaultCollisionHandler(self._space)
        h = CollisionHandler(_h, self)
        self._handlers[None] = h
        return h

    def add_post_step_callback(self, callback_function, key, *args, **kwargs):
        """Add a function to be called last in the next simulation step.

        Post step callbacks are registered as a function and an object used as
        a key. You can only register one post step callback per object.

        This function was more useful with earlier versions of pymunk where
        you weren't allowed to use the add and remove methods on the space
        during a simulation step. But this function is still available for
        other uses and to keep backwards compatibility.

        .. Note::
            If you remove a shape from the callback it will trigger the
            collision handler for the 'separate' event if it the shape was
            touching when removed.

        .. Note::
            Post step callbacks are not included in pickle / copy of the space.

        :param callback_function: The callback function
        :type callback_function: `func(space : Space, key, *args, **kwargs)`
        :param Any key: 
            This object is used as a key, you can only have one callback
            for a single object. It is passed on to the callback function.
        :param args: Optional parameters passed to the callback
        :param kwargs: Optional keyword parameters passed on to the callback    

        :return: True if key was not previously added, False otherwise
        """

        if key in self._post_step_callbacks:
            return False
        
        def f(x):
            callback_function(self, key, *args, **kwargs)
            
        self._post_step_callbacks[key] = f
        return True
        
    def point_query(self, point, max_distance, shape_filter):
        """Query space at point for shapes within the given distance range.

        The filter is applied to the query and follows the same rules as the
        collision detection. If a maxDistance of 0.0 is used, the point must 
        lie inside a shape. Negative max_distance is also allowed meaning that 
        the point must be a under a certain depth within a shape to be 
        considered a match.

        See :py:class:`ShapeFilter` for details about how the shape_filter 
        parameter can be used.

        .. Note::
            Sensor shapes are included in the result (In 
            :py:meth:`Space.point_query_nearest` they are not)

        :param point: Where to check for collision in the Space
        :type point: :py:class:`~vec2d.Vec2d` or (float,float)
        :param float max_distance: Match only within this distance
        :param ShapeFilter shape_filter: Only pick shapes matching the filter

        :rtype: [:py:class:`PointQueryInfo`] 
        """

        self.__query_hits = []
        
        @ffi.callback("cpSpacePointQueryFunc")
        def cf(_shape, point, distance, gradient, data):
            # space = ffi.from_handle(data)
            shape = self._get_shape(_shape)
            p = PointQueryInfo(
                shape, 
                Vec2d._fromcffi(point), 
                distance, 
                Vec2d._fromcffi(gradient))
            self.__query_hits.append(p)
            
        data = ffi.new_handle(self)
        cp.cpSpacePointQuery(
            self._space, tuple(point), max_distance, shape_filter, cf, data)
        return self.__query_hits

    def _get_shape(self, _shape):
        if not bool(_shape):
            return None

        shapeid = cp.cpShapeGetUserData(_shape)
        #return self._shapes[hashid_private]
        if shapeid in self._shapes:
            shape = self._shapes[shapeid]
        elif shapeid in self._removed_shapes:
            shape = self._removed_shapes[shapeid]
        else:
            shape = None
        return shape

    def point_query_nearest(self, point, max_distance, shape_filter):
        """Query space at point the nearest shape within the given distance
        range.

        The filter is applied to the query and follows the same rules as the
        collision detection. If a maxDistance of 0.0 is used, the point must 
        lie inside a shape. Negative max_distance is also allowed meaning that 
        the point must be a under a certain depth within a shape to be 
        considered a match.

        See :py:class:`ShapeFilter` for details about how the shape_filter 
        parameter can be used.

        .. Note::
            Sensor shapes are not included in the result (In 
            :py:meth:`Space.point_query` they are)

        :param point: Where to check for collision in the Space
        :type point: :py:class:`~vec2d.Vec2d` or (float,float)
        :param float max_distance: Match only within this distance
        :param ShapeFilter shape_filter: Only pick shapes matching the filter

        :rtype: :py:class:`PointQueryInfo` or None  
        """
        info = ffi.new("cpPointQueryInfo *")
        _shape = cp.cpSpacePointQueryNearest(
            self._space, 
            tuple(point), max_distance, 
            shape_filter, info)
        
        shape = self._get_shape(_shape)

        if shape != None:
            return PointQueryInfo(
                shape, 
                Vec2d._fromcffi(info.point), 
                info.distance, 
                Vec2d._fromcffi(info.gradient))
        return None


    def segment_query(self, start, end, radius, shape_filter):
        """Query space along the line segment from start to end with the
        given radius.

        The filter is applied to the query and follows the same rules as the
        collision detection.

        See :py:class:`ShapeFilter` for details about how the shape_filter 
        parameter can be used.

        .. Note::
            Sensor shapes are included in the result (In 
            :py:meth:`Space.segment_query_first` they are not)

        :param start: Starting point
        :param end: End point
        :param float radius: Radius
        :param ShapeFilter shape_filter: Shape filter

        :rtype: [:py:class:`SegmentQueryInfo`]
        """

        self.__query_hits = []
        
        @ffi.callback("cpSpaceSegmentQueryFunc")
        def cf(_shape, point, normal, alpha, data):
            shape = self._get_shape(_shape)
            p = SegmentQueryInfo(
                shape, Vec2d._fromcffi(point), Vec2d._fromcffi(normal), alpha)
            self.__query_hits.append(p)
            
        data = ffi.new_handle(self)
        cp.cpSpaceSegmentQuery(
            self._space, 
            tuple(start), tuple(end), 
            radius, 
            shape_filter, cf, 
            data)
        return self.__query_hits

    def segment_query_first(self, start, end, radius, shape_filter):
        """Query space along the line segment from start to end with the
        given radius.

        The filter is applied to the query and follows the same rules as the
        collision detection.

        .. Note::
            Sensor shapes are not included in the result (In 
            :py:meth:`Space.segment_query` they are)

        See :py:class:`ShapeFilter` for details about how the shape_filter 
        parameter can be used.

        :rtype: :py:class:`SegmentQueryInfo` or None
        """
        info = ffi.new("cpSegmentQueryInfo *")
        _shape = cp.cpSpaceSegmentQueryFirst(
            self._space, 
            tuple(start), tuple(end), 
            radius, shape_filter, 
            info)

        shape = self._get_shape(_shape)
        if shape != None:
            return SegmentQueryInfo(
                shape, 
                Vec2d._fromcffi(info.point), 
                Vec2d._fromcffi(info.normal), 
                info.alpha)
        return None


    def bb_query(self, bb, shape_filter):
        """Query space to find all shapes near bb.

        The filter is applied to the query and follows the same rules as the
        collision detection.

        .. Note::
            Sensor shapes are included in the result

        :param BB bb: Bounding box
        :param ShapeFilter shape_filter: Shape filter

        :rtype: [:py:class:`Shape`]
        """

        self.__query_hits = []
        @ffi.callback("cpSpaceBBQueryFunc")
        def cf(_shape, data):
            shape = self._get_shape(_shape)
            self.__query_hits.append(shape)
        
        data = ffi.new_handle(self)
        cp.cpSpaceBBQuery(self._space, bb._bb, shape_filter, cf, data)
        return self.__query_hits


    def shape_query(self, shape):
        """Query a space for any shapes overlapping the given shape

        .. Note::
            Sensor shapes are included in the result

        :param shape: Shape to query with
        :type shape: :py:class:`Circle`, :py:class:`Poly` or :py:class:`Segment`

        :rtype: [:py:class:`ShapeQueryInfo`]
        """

        self.__query_hits = []
        @ffi.callback("cpSpaceShapeQueryFunc")
        def cf(_shape, _points, _data):
            shape = self._get_shape(_shape)
            point_set = ContactPointSet._from_cp(_points)
            info = ShapeQueryInfo(shape, point_set)
            self.__query_hits.append(info)
            
        data = ffi.new_handle(self)
        cp.cpSpaceShapeQuery(self._space, shape._shape, cf, data)

        return self.__query_hits


    def debug_draw(self, options):
        """Debug draw the current state of the space using the supplied drawing 
        options.        

        If you use a graphics backend that is already supported, such as pygame 
        and pyglet, you can use the predefined options in their x_util modules, 
        for example :py:class:`pygame_util.DrawOptions`.

        Its also possible to write your own graphics backend, see 
        :py:class:`SpaceDebugDrawOptions`.

        If you require any advanced or optimized drawing its probably best to 
        not use this function for the drawing since its meant for debugging 
        and quick scripting. 

        :type options: :py:class:`SpaceDebugDrawOptions`
        """
        if options._use_chipmunk_debug_draw:
            h = ffi.new_handle(self)
            # we need to hold h until the end of cpSpaceDebugDraw to prevent GC
            options._options.data = h
            
            with options:
                cp.cpSpaceDebugDraw(self._space, options._options)
        else:
            for shape in self.shapes:
                options.draw_shape(shape)
        
    def __getstate__(self):
        """Return the state of this object
        
        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        d = super(Space, self).__getstate__()

        d['special'].append(('shapes', self.shapes))
        d['special'].append(('bodies', self.bodies))
        d['special'].append(('constraints', self.constraints))
        if self._static_body != None:
            d['special'].append(('_static_body', self._static_body))
        
        handlers = []
        for k,v in self._handlers.items():
            h = {}
            if v._begin_base != None:
                h['_begin_base'] = v._begin_base
            if v._pre_solve_base != None:
                h['_pre_solve_base'] = v._pre_solve_base
            if v._post_solve_base != None:
                h['_post_solve_base'] = v._post_solve_base
            if v._separate_base != None:
                h['_separate_base'] = v._separate_base
            handlers.append((k,h))
            
        d['special'].append(('_handlers', handlers))
            
        return d

    def __setstate__(self, state):
        """Unpack this object from a saved state.

        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        super(Space, self).__setstate__(state)
        
        for k,v in state['special']:
            if k == 'shapes':
                self.add(*v)
            if k == 'bodies':
                self.add(*v)
            if k == 'constraints':
                self.add(*v)
            if k == '_static_body':
                self._static_body = v
                self._static_body._space = self
            if k == '_handlers':
                for k, hd in v:
                    if k == None:
                        h = self.add_default_collision_handler()
                    elif isinstance(k,tuple):
                        h = self.add_collision_handler(k[0], k[1])
                    else:
                        h = self.add_wildcard_collision_handler(k)
                    if '_begin_base' in hd:
                        h.begin = hd['_begin_base'] 
                    if '_pre_solve_base' in hd:
                        h.pre_solve = hd['_pre_solve_base'] 
                    if '_post_solve_base' in hd:
                        h.post_solve = hd['_post_solve_base'] 
                    if '_separate_base' in hd:
                        h.separate = hd['_separate_base'] 

    def copy(self):
        """Create a deep copy of this space."""
        return copy.deepcopy(self)



__docformat__ = "reStructuredText"

from .vec2d import Vec2d

class ContactPoint(object):
    """Contains information about a contact point. 
    
    point_a and point_b are the contact position on the surface of each shape.
    
    distance is the penetration distance of the two shapes. Overlapping 
    means it will be negative. This value is calculated as 
    dot(point2 - point1), normal) and is ignored when you set the 
    Arbiter.contact_point_set.
    """
    
    __slots__ = ('point_a', 'point_b', 'distance')
    
    def __init__(self, point_a, point_b, distance):
        self.point_a = point_a
        self.point_b = point_b
        self.distance = distance
    
    def __repr__(self):
        return 'ContactPoint(point_a={}, point_b={}, distance={})'.format(
            self.point_a, self.point_b, self.distance)
    

class ContactPointSet(object):
    """Contact point sets make getting contact information simpler.
    
    normal is the normal of the collision
    
    points is the array of contact points. Can be at most 2 points.
    """
    
    __slots__ = ('normal', 'points')
    
    def __init__(self, normal, points):
        self.normal = normal
        self.points = points
    
    def __repr__(self):
        return 'ContactPointSet(normal={}, points={})'.format(
            self.normal, self.points)
    
    @classmethod
    def _from_cp(cls, _points):
        normal = Vec2d(_points.normal)
        
        points = [] 
        for i in range(_points.count):
            _p = _points.points[i]
            p = ContactPoint(
                Vec2d._fromcffi(_p.pointA), 
                Vec2d._fromcffi(_p.pointB), 
                _p.distance)
            points.append(p)
        
        return cls(normal, points)



from collections import namedtuple

class Transform(namedtuple('Transform', ['a','b','c','d','tx','ty'])):
    """Type used for 2x3 affine transforms.
    
    See wikipedia for details: 
    http://en.wikipedia.org/wiki/Affine_transformation
    
    The properties map to the matrix in this way: 
    
    = = ==
    = = ==
    a c tx
    b d ty
    = = ==
        
    An instance can be created in this way::
        >>> Transform(1,2,3,4,5,6)
        Transform(a=1, b=2, c=3, d=4, tx=5, ty=6)
    Or using the default identity in this way::
        >>> Transform.identity()
        Transform(a=1, b=0, c=0, d=1, tx=0, ty=0)
    Or overriding only some of the values (on a identity matrix):
        >>> Transform(b=3,ty=5)
        Transform(a=1, b=3, c=0, d=1, tx=0, ty=5)
    
        
    """
    __slots__ = ()
    
    def __new__(cls, a=1, b=0, c=0, d=1, tx=0, ty=0):
        self = super(Transform, cls).__new__(cls, a, b, c, d, tx, ty)
        return self
    
    @staticmethod
    def identity():
        """The identity transform"""
        return Transform(1,0,0,1,0,0)



import unittest
import doctest
import sys
import os

def main():
    
    def list_of_tests_gen(s):
        for test in s:
            if unittest.suite._isnotsuite(test):
                yield test
            else:
                for t in list_of_tests_gen(test):
                    yield t

    from . import doctests
    
    path = os.path.dirname(os.path.abspath(__file__))
    suite = unittest.TestLoader().discover(path)

    doctests.load_tests(None, suite, None)

    wasSuccessful = True

    filtered_suite = unittest.TestSuite()

    if len(sys.argv) > 1:
        test_filter = sys.argv[1]
        for test in list_of_tests_gen(suite):
            if isinstance(test, doctest.DocTestCase) \
                and test_filter.startswith("doctest") \
                or test_filter in str(test.id()):
                filtered_suite.addTest(test)

    elif sys.version_info.major < 3:
        print("Skipping doctests"
              " (Doctests uses on formatting that differ between Py2 and 3)")
        for test in list_of_tests_gen(suite):
            if not isinstance(test, doctest.DocTestCase):
                filtered_suite.addTest(test)
    else:
        filtered_suite = suite
    
    res = unittest.TextTestRunner(verbosity=2).run(filtered_suite)
    wasSuccessful = res.wasSuccessful()

    sys.exit(not wasSuccessful)
    
if __name__ == '__main__':
    import os
    import sys
    #sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
    main()


class PickleMixin(object):
    """PickleMixin is used to provide base functionality for pickle/unpickle 
    and copy.
    """

    def __getstate__(self):
        """Return the state of this object
        
        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        
        attrs_init = []
        attrs_general = []
        for t in list(type(self).__bases__) + [type(self)]:
            if hasattr(t, '_pickle_attrs_init'):
                attrs_init += t._pickle_attrs_init
            if hasattr(t, '_pickle_attrs_general'):
                attrs_general += t._pickle_attrs_general
            
        d = {
            'init':[], # arguments for init
            'general':[], # general attributes 
            'custom':[], # custom attributes set by user
            'special':[] # attributes needing special handling
        }
        for a in attrs_init:
            d['init'].append((a, self.__getattribute__(a)))

        for a in attrs_general:
            d['general'].append((a, self.__getattribute__(a)))
        
        for k,v in self.__dict__.items():
            if k[0] != '_':
                d['custom'].append((k,v))
        
        return d

    def __setstate__(self, state):
        """Unpack this object from a saved state.

        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        
        init_attrs = []

        init_args = [v for k,v in state['init']]
        self.__init__(*init_args)

        for k,v in state['general']:
            self.__setattr__(k,v)

        for k,v in state['custom']:
            self.__setattr__(k,v)

        


import pymunk as p
from pymunk.vec2d import Vec2d
import unittest
import pickle

class UnitTestBB(unittest.TestCase):
    def setUp(self):
        #print "testing pymunk version " + p.version
        pass

    def testCreation(self):
        bb_empty = p.BB()

        self.assertEqual(bb_empty.left, 0)
        self.assertEqual(bb_empty.bottom, 0)
        self.assertEqual(bb_empty.right, 0)
        self.assertEqual(bb_empty.top , 0)

        bb_defined = p.BB(-10,-5,15,20)

        self.assertEqual(bb_defined.left, -10)
        self.assertEqual(bb_defined.bottom, -5)
        self.assertEqual(bb_defined.right, 15)
        self.assertEqual(bb_defined.top, 20)

        bb_circle = p.BB.newForCircle((3,3),3)
        self.assertEqual(bb_circle.left, 0)
        self.assertEqual(bb_circle.bottom, 0)
        self.assertEqual(bb_circle.right, 6)
        self.assertEqual(bb_circle.top, 6)

    def testMerge(self):
        bb1 = p.BB(0,0,10,10)
        bb2 = p.BB(2,0,10,10)
        bb3 = p.BB(10,10,15,15)

        self.assertEqual(bb1.merge(bb2), p.BB(0,0,10,10))
        self.assertEqual(bb2.merge(bb3).merge(bb1), p.BB(0,0,15,15))

    def testMethods(self):
        bb1 = p.BB(0,0,10,10)
        bb2 = p.BB(10,10,20,20)
        bb3 = p.BB(4,4,5,5)
        bb4 = p.BB(2,0,10,10)

        v1 = Vec2d(1,1)
        v2 = Vec2d(100,3)
        self.assertTrue(bb1.intersects(bb2))
        self.assertFalse(bb3.intersects(bb2))

        self.assertTrue(bb1.intersects_segment(v1,v2))
        self.assertFalse(bb3.intersects_segment(v1,v2))

        self.assertTrue(bb1.contains(bb3))
        self.assertFalse(bb1.contains(bb2))

        self.assertTrue(bb1.contains_vect(v1))
        self.assertFalse(bb1.contains_vect(v2))

        self.assertEqual(bb1.expand(v1), bb1)
        self.assertEqual(bb1.expand(-v2), p.BB(-100,-3,10,10))

        self.assertEqual(bb1.center(), (5,5))
        self.assertEqual(bb1.area(), 100)

        self.assertEqual(bb1.merged_area(bb2), 400)

        self.assertEqual(bb2.segment_query(v1, v2), p.inf)
        self.assertEqual(bb1.segment_query((-1,1), (99,1)), 0.01)


        self.assertEqual(bb1.clamp_vect(v2), Vec2d(10,3))

        #self.assertEqual(bb1.wrap_vect((11,11)), (1,1))

    def testPickle(self):
        x = p.BB(4,4,5,5)

        s = pickle.dumps(x, 2)
        actual = pickle.loads(s)

        self.assertEqual(x, actual)

        x2 = x.copy()



__docformat__ = "reStructuredText"

import copy

from . import _chipmunk_cffi
lib = _chipmunk_cffi.lib
ffi = _chipmunk_cffi.ffi
from .vec2d import Vec2d
from ._pickle import PickleMixin

class BB(PickleMixin, object):
    """Simple bounding box.

    Stored as left, bottom, right, top values.
    """

    _pickle_attrs_init = ['left', 'bottom', 'right', 'top']

    def __init__(self, *args):
        """Create a new instance of a bounding box. 
        
        Can be created with zero size with bb = BB() or with four args defining 
        left, bottom, right and top: bb = BB(left, bottom, right, top)
        """
        if len(args) == 0:
            self._bbp = ffi.new("cpBB *")
            self._bb = self._bbp[0]
        elif len(args) == 1:
            self._bb = args[0]
        else:
            self._bbp = ffi.new("cpBB *", args)
            self._bb = self._bbp[0]

    @staticmethod
    def newForCircle(p, r):
        """Convenience constructor for making a BB fitting a circle at
        position p with radius r.
        """
        
        bb_ = lib._cpBBNewForCircle(p, r)
        return BB(bb_)

    def __repr__(self):
        return 'BB(%s, %s, %s, %s)' % (self.left, self.bottom, self.right, self.top)

    def __eq__(self, other):
        return self.left == other.left and self.bottom == other.bottom and \
            self.right == other.right and self.top == other.top

    def __ne__(self, other):
        return not self.__eq__(other)

    left = property(lambda self: self._bb.l)
    bottom = property(lambda self: self._bb.b)
    right = property(lambda self: self._bb.r)
    top = property(lambda self: self._bb.t)

    def intersects(self, other):
        """Returns true if the bounding boxes intersect"""
        return bool(lib._cpBBIntersects(self._bb, other._bb))

    def intersects_segment(self, a, b):
        """Returns true if the segment defined by endpoints a and b
        intersect this bb."""
        return bool(lib._cpBBIntersectsSegment(self._bb, tuple(a), tuple(b)))

    def contains(self, other):
        """Returns true if bb completley contains the other bb"""
        return bool(lib._cpBBContainsBB(self._bb, other._bb))

    def contains_vect(self, v):
        """Returns true if this bb contains the vector v"""
        return bool(lib._cpBBContainsVect(self._bb, tuple(v)))

    def merge(self, other):
        """Return the minimal bounding box that contains both this bb and the
        other bb
        """
        return BB(lib._cpBBMerge(self._bb, other._bb))

    def expand(self, v):
        """Return the minimal bounding box that contans both this bounding box
        and the vector v
        """
        return BB(lib._cpBBExpand(self._bb, tuple(v)))

    def center(self):
        """Return the center"""
        return Vec2d._fromcffi(lib._cpBBCenter(self._bb))

    def area(self):
        """Return the area"""
        return lib._cpBBArea(self._bb)

    def merged_area(self, other):
        """Merges this and other then returns the area of the merged bounding
        box.
        """
        return lib._cpBBMergedArea(self._bb, other._bb)

    def segment_query(self, a, b):
        """Returns the fraction along the segment query the BB is hit.

        Returns infinity if it doesnt hit
        """
        return lib._cpBBSegmentQuery(self._bb, tuple(a), tuple(b))

    def clamp_vect(self, v):
        """Returns a copy of the vector v clamped to the bounding box"""
        return Vec2d._fromcffi(lib._cpBBClampVect(self._bb, tuple(v)))

    '''
    def wrap_vect(self, v):
        """Returns a copy of v wrapped to the bounding box.

        That is, BB(0,0,10,10).wrap_vect((5,5)) == Vec2d._fromcffi(10,10)
        """
        return lib._cpBBWrapVect(self._bb[0], v)
    '''

    def copy(self):
        """Create a deep copy of this BB."""
        return copy.deepcopy(self)



# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2011 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""This module contain the Vec2d class that is used in all of pymunk when a 
vector is needed.

The Vec2d class is used almost everywhere in pymunk for 2d coordinates and 
vectors, for example to define gravity vector in a space. However, pymunk is 
smart enough to convert tuples or tuple like objects to Vec2ds so you usually 
do not need to explicitly do conversions if you happen to have a tuple::

    >>> import pymunk
    >>> space = pymunk.Space()
    >>> space.gravity
    Vec2d(0.0, 0.0)
    >>> space.gravity = 3,5
    >>> space.gravity
    Vec2d(3.0, 5.0)
    >>> space.gravity += 2,6
    >>> space.gravity
    Vec2d(5.0, 11.0)
    
More examples::

    >>> from pymunk.vec2d import Vec2d
    >>> Vec2d(7.3, 4.2)
    Vec2d(7.3, 4.2)
    >>> Vec2d((7.3, 4.2))
    Vec2d(7.3, 4.2)
    >>> Vec2d(7.3, 4.2) + Vec2d((1,2))
    Vec2d(8.3, 6.2)
    
"""
__docformat__ = "reStructuredText"

import operator
import math

__all__ = ["Vec2d"]

class Vec2d(object):
    """2d vector class, supports vector and scalar operators, and also 
    provides some high level functions.
    """
    __slots__ = ("x", "y")

    @staticmethod        
    def _fromcffi(p):
        """Used as a speedy way to create Vec2ds internally in pymunk."""
        v = Vec2d.__new__(Vec2d)
        v.x = p.x
        v.y = p.y
        return v 

    def __init__(self, x_or_pair=None, y = None):
        if x_or_pair != None:
            if y == None:
                if hasattr(x_or_pair, "x") and hasattr(x_or_pair, "y"):
                    self.x = x_or_pair.x
                    self.y = x_or_pair.y
                else:
                    self.x = x_or_pair[0]
                    self.y = x_or_pair[1]
            else:
                self.x = x_or_pair
                self.y = y
        else:
            self.x = 0
            self.y = 0

    def __getitem__(self, i):
        if i == 0:
            return self.x
        elif i == 1:
            return self.y
        raise IndexError()

    def __iter__(self):
        yield self.x
        yield self.y

    def __len__(self):
        return 2

    def __setitem__(self, i, value):
        if i == 0:
            self.x = value
        elif i == 1:
            self.y = value
        else:
            raise IndexError()
   
    # String representaion (for debugging)
    def __repr__(self):
        return 'Vec2d(%s, %s)' % (self.x, self.y)
    
    # Comparison
    def __eq__(self, other):
        if hasattr(other, "__getitem__") and len(other) == 2:
            return self.x == other[0] and self.y == other[1]
        else:
            return False
    
    def __ne__(self, other):
        if hasattr(other, "__getitem__") and len(other) == 2:
            return self.x != other[0] or self.y != other[1]
        else:
            return True
 
    def __nonzero__(self):
        return self.x != 0.0 or self.y != 0.0
 
    # Generic operator handlers
    def _o2(self, other, f):
        "Any two-operator operation where the left operand is a Vec2d"
        if isinstance(other, Vec2d):
            return Vec2d(f(self.x, other.x),
                         f(self.y, other.y))
        elif (hasattr(other, "__getitem__")):
            return Vec2d(f(self.x, other[0]),
                         f(self.y, other[1]))
        else:
            return Vec2d(f(self.x, other),
                         f(self.y, other))
 
    def _r_o2(self, other, f):
        "Any two-operator operation where the right operand is a Vec2d"
        if (hasattr(other, "__getitem__")):
            return Vec2d(f(other[0], self.x),
                         f(other[1], self.y))
        else:
            return Vec2d(f(other, self.x),
                         f(other, self.y))
 
    def _io(self, other, f):
        "inplace operator"
        if (hasattr(other, "__getitem__")):
            self.x = f(self.x, other[0])
            self.y = f(self.y, other[1])
        else:
            self.x = f(self.x, other)
            self.y = f(self.y, other)
        return self
 
    # Addition
    def __add__(self, other):
        if isinstance(other, Vec2d):
            return Vec2d(self.x + other.x, self.y + other.y)
        elif hasattr(other, "__getitem__"):
            return Vec2d(self.x + other[0], self.y + other[1])
        else:
            return Vec2d(self.x + other, self.y + other)
    __radd__ = __add__
    
    def __iadd__(self, other):
        if isinstance(other, Vec2d):
            self.x += other.x
            self.y += other.y
        elif hasattr(other, "__getitem__"):
            self.x += other[0]
            self.y += other[1]
        else:
            self.x += other
            self.y += other
        return self
 
    # Subtraction
    def __sub__(self, other):
        if isinstance(other, Vec2d):
            return Vec2d(self.x - other.x, self.y - other.y)
        elif (hasattr(other, "__getitem__")):
            return Vec2d(self.x - other[0], self.y - other[1])
        else:
            return Vec2d(self.x - other, self.y - other)
    def __rsub__(self, other):
        if isinstance(other, Vec2d):
            return Vec2d(other.x - self.x, other.y - self.y)
        if (hasattr(other, "__getitem__")):
            return Vec2d(other[0] - self.x, other[1] - self.y)
        else:
            return Vec2d(other - self.x, other - self.y)
    def __isub__(self, other):
        if isinstance(other, Vec2d):
            self.x -= other.x
            self.y -= other.y
        elif (hasattr(other, "__getitem__")):
            self.x -= other[0]
            self.y -= other[1]
        else:
            self.x -= other
            self.y -= other
        return self
 
    # Multiplication
    def __mul__(self, other):
        if isinstance(other, Vec2d):
            return Vec2d(self.x*other.x, self.y*other.y)
        if (hasattr(other, "__getitem__")):
            return Vec2d(self.x*other[0], self.y*other[1])
        else:
            return Vec2d(self.x*other, self.y*other)
    __rmul__ = __mul__
    
    def __imul__(self, other):
        if isinstance(other, Vec2d):
            self.x *= other.x
            self.y *= other.y
        elif (hasattr(other, "__getitem__")):
            self.x *= other[0]
            self.y *= other[1]
        else:
            self.x *= other
            self.y *= other
        return self
 
    # Division
    def __div__(self, other):
        return self._o2(other, operator.div)
    def __rdiv__(self, other):
        return self._r_o2(other, operator.div)
    def __idiv__(self, other):
        return self._io(other, operator.div)
 
    def __floordiv__(self, other):
        return self._o2(other, operator.floordiv)
    def __rfloordiv__(self, other):
        return self._r_o2(other, operator.floordiv)
    def __ifloordiv__(self, other):
        return self._io(other, operator.floordiv)
 
    def __truediv__(self, other):
        return self._o2(other, operator.truediv)
    def __rtruediv__(self, other):
        return self._r_o2(other, operator.truediv)
    def __itruediv__(self, other):
        return self._io(other, operator.truediv)
 
    # Modulo
    def __mod__(self, other):
        return self._o2(other, operator.mod)
    def __rmod__(self, other):
        return self._r_o2(other, operator.mod)
 
    def __divmod__(self, other):
        return self._o2(other, divmod)
    def __rdivmod__(self, other):
        return self._r_o2(other, divmod)
 
    # Exponentation
    def __pow__(self, other):
        return self._o2(other, operator.pow)
    def __rpow__(self, other):
        return self._r_o2(other, operator.pow)
 
    # Bitwise operators
    def __lshift__(self, other):
        return self._o2(other, operator.lshift)
    def __rlshift__(self, other):
        return self._r_o2(other, operator.lshift)
 
    def __rshift__(self, other):
        return self._o2(other, operator.rshift)
    def __rrshift__(self, other):
        return self._r_o2(other, operator.rshift)
 
    def __and__(self, other):
        return self._o2(other, operator.and_)
    __rand__ = __and__
 
    def __or__(self, other):
        return self._o2(other, operator.or_)
    __ror__ = __or__
 
    def __xor__(self, other):
        return self._o2(other, operator.xor)
    __rxor__ = __xor__
 
    # Unary operations
    def __neg__(self):
        return Vec2d(operator.neg(self.x), operator.neg(self.y))
 
    def __pos__(self):
        return Vec2d(operator.pos(self.x), operator.pos(self.y))
 
    def __abs__(self):
        return Vec2d(abs(self.x), abs(self.y))
 
    def __invert__(self):
        return Vec2d(-self.x, -self.y)
 
    
   
 
    # vectory functions
    def get_length_sqrd(self): 
        """Get the squared length of the vector.
        It is more efficient to use this method instead of first call 
        get_length() or access .length and then do a sqrt().
        
        :return: The squared length
        """
        return self.x**2 + self.y**2
 
    def get_length(self):
        """Get the length of the vector.
        
        :return: The length
        """
        return math.sqrt(self.x**2 + self.y**2)    
    def __setlength(self, value):
        length = self.get_length()
        self.x *= value/length
        self.y *= value/length
    length = property(get_length, __setlength, 
        doc = """Gets or sets the magnitude of the vector""")
       
    def rotate(self, angle_radians):
        """Rotate the vector by angle_radians radians."""
        cos = math.cos(angle_radians)
        sin = math.sin(angle_radians)
        x = self.x*cos - self.y*sin
        y = self.x*sin + self.y*cos
        self.x = x
        self.y = y
 
    def rotated(self, angle_radians):
        """Create and return a new vector by rotating this vector by 
        angle_radians radians.
        
        :return: Rotated vector
        """
        cos = math.cos(angle_radians)
        sin = math.sin(angle_radians)
        x = self.x*cos - self.y*sin
        y = self.x*sin + self.y*cos
        return Vec2d(x, y)
    
    def rotate_degrees(self, angle_degrees):
        """Rotate the vector by angle_degrees degrees."""
        self.rotate(math.radians(angle_degrees))
    
    def rotated_degrees(self, angle_degrees):
        """Create and return a new vector by rotating this vector by 
        angle_degrees degrees.
        
        :return: Rotade vector
        """
        return self.rotated(math.radians(angle_degrees))
    
    def get_angle(self):
        if (self.get_length_sqrd() == 0):
            return 0
        return math.atan2(self.y, self.x)
    def __setangle(self, angle):
        self.x = self.length
        self.y = 0
        self.rotate(angle)
    angle = property(get_angle, __setangle, 
        doc="""Gets or sets the angle (in radians) of a vector""")
 
    def get_angle_degrees(self):
        return math.degrees(self.get_angle())
    def __set_angle_degrees(self, angle_degrees):
        self.__setangle(math.radians(angle_degrees))
    angle_degrees = property(get_angle_degrees, __set_angle_degrees, 
        doc="""Gets or sets the angle (in degrees) of a vector""")
    
    def get_angle_between(self, other):
        """Get the angle between the vector and the other in radians
        
        :return: The angle
        """
        cross = self.x*other[1] - self.y*other[0]
        dot = self.x*other[0] + self.y*other[1]
        return math.atan2(cross, dot)
        
    def get_angle_degrees_between(self, other):
        """Get the angle between the vector and the other in degrees
        
        :return: The angle (in degrees)
        """
        return math.degrees(self.get_angle_between(other))
        
    def normalized(self):
        """Get a normalized copy of the vector
        Note: This function will return 0 if the length of the vector is 0.
        
        :return: A normalized vector
        """
        length = self.length
        if length != 0:
            return self/length
        return Vec2d(self)
 
    def normalize_return_length(self):
        """Normalize the vector and return its length before the normalization
        
        :return: The length before the normalization
        """
        length = self.length
        if length != 0:
            self.x /= length
            self.y /= length
        return length
 
    def perpendicular(self):
        return Vec2d(-self.y, self.x)
    
    def perpendicular_normal(self):
        length = self.length
        if length != 0:
            return Vec2d(-self.y/length, self.x/length)
        return Vec2d(self)
        
    def dot(self, other):
        """The dot product between the vector and other vector
            v1.dot(v2) -> v1.x*v2.x + v1.y*v2.y
            
        :return: The dot product
        """
        return float(self.x*other[0] + self.y*other[1])
        
    def get_distance(self, other):
        """The distance between the vector and other vector
        
        :return: The distance
        """
        return math.sqrt((self.x - other[0])**2 + (self.y - other[1])**2)
        
    def get_dist_sqrd(self, other):
        """The squared distance between the vector and other vector
        It is more efficent to use this method than to call get_distance()
        first and then do a sqrt() on the result.
        
        :return: The squared distance
        """
        return (self.x - other[0])**2 + (self.y - other[1])**2
        
    def projection(self, other):
        other_length_sqrd = other[0]*other[0] + other[1]*other[1]
        projected_length_times_other_length = self.dot(other)
        return other*(projected_length_times_other_length/other_length_sqrd)
    
    def cross(self, other):
        """The cross product between the vector and other vector
            v1.cross(v2) -> v1.x*v2.y - v1.y*v2.x
        
        :return: The cross product
        """
        return self.x*other[1] - self.y*other[0]
    
    def interpolate_to(self, other, range):
        return Vec2d(self.x + (other[0] - self.x)*range, self.y + (other[1] - self.y)*range)
    
    def convert_to_basis(self, x_vector, y_vector):
        x = self.dot(x_vector)/x_vector.get_length_sqrd()
        y = self.dot(y_vector)/y_vector.get_length_sqrd()
        return Vec2d(x, y)
    
    def __get_int_xy(self):
        return int(self.x), int(self.y)
    int_tuple = property(__get_int_xy, 
        doc="""Return the x and y values of this vector as ints""")
    
    @staticmethod
    def zero():
        """A vector of zero length"""
        return Vec2d(0, 0)
        
    @staticmethod
    def unit():
        """A unit vector pointing up"""
        return Vec2d(0, 1)
        
    @staticmethod
    def ones():
        """A vector where both x and y is 1"""
        return Vec2d(1, 1)
 
    # Extra functions, mainly for chipmunk
    def cpvrotate(self, other):
        """Uses complex multiplication to rotate this vector by the other. """
        return Vec2d(self.x*other.x - self.y*other.y, self.x*other.y + self.y*other.x)
    def cpvunrotate(self, other):
        """The inverse of cpvrotate"""
        return Vec2d(self.x*other.x + self.y*other.y, self.y*other.x - self.x*other.y)
    
    # Pickle
    def __reduce__(self):
        callable = Vec2d
        args = (self.x, self.y)
        return (callable, args)
    





"""This submodule contains helper functions to help with quick prototyping 
using pymunk together with pyglet.

Intended to help with debugging and prototyping, not for actual production use
in a full application. The methods contained in this module is opinionated 
about your coordinate system and not very optimized (they use batched 
drawing, but there is probably room for optimizations still). 
"""

__docformat__ = "reStructuredText"

__all__ = ["DrawOptions"]

import matplotlib.pyplot as plt

import pymunk
from pymunk.vec2d import Vec2d

class DrawOptions(pymunk.SpaceDebugDrawOptions):
    def __init__(self, ax):
        """DrawOptions for space.debug_draw() to draw a space on a ax object. 

        Typical usage::
        
        >>> import matplotlib as mpl
        >>> import pymunk
        >>> import pymunk.matplotlib_util
        >>> my_space = pymunk.Space()
        >>> fix, ax = mpl.subplot()
        >>> options = pymunk.matplotlib_util.DrawOptions(ax)
        >>> my_space.debug_draw(options)

        You can control the color of a Shape by setting shape.color to the color 
        you want it drawn in.

        >>> my_shape.color = (1, 0, 0, 1) # will draw my_shape in red

        See matplotlib_util.demo.py for a full example

        :Param:
            ax: matplotlib.Axes
                A matplotlib Axes object.

        """
        super(DrawOptions, self).__init__()

        self.ax = ax
        self.shape_dynamic_color = self.shape_dynamic_color.as_float()
        self.shape_static_color = self.shape_static_color.as_float()
        self.shape_kinematic_color = self.shape_kinematic_color.as_float()
        self.shape_sleeping_color = self.shape_sleeping_color.as_float()
        self.shape_outline_color = self.shape_outline_color.as_float()
        self.constraint_color = self.constraint_color.as_float()
        self.collision_point_color = self.collision_point_color.as_float()
    
    def draw_circle(self, pos, angle, radius, outline_color, fill_color):        
        p = plt.Circle(pos, radius, 
            facecolor=fill_color, edgecolor=outline_color)
        self.ax.add_patch(p)
        
        circle_edge = pos + Vec2d(radius, 0).rotated(angle)
        line = plt.Line2D([pos.x, circle_edge.x], [pos.y, circle_edge.y], 
            linewidth=1, color=outline_color)
        line.set_solid_capstyle("round")
        self.ax.add_line(line)

    def draw_segment(self, a, b, color):
        line = plt.Line2D([a.x, b.x], [a.y, b.y], linewidth=1, color=color)
        line.set_solid_capstyle("round")
        self.ax.add_line(line)

    def draw_fat_segment(self, a, b, radius, outline_color, fill_color):
        radius = max(1, 2*radius)
        line = plt.Line2D([a.x, b.x], [a.y, b.y], 
            linewidth=radius, color=fill_color)
        line.set_solid_capstyle("round")
        self.ax.add_line(line)
        
    def draw_polygon(self, verts, radius, outline_color, fill_color):
        radius = max(1, 2*radius)
        p = plt.Polygon(verts, linewidth=radius, joinstyle="round", 
            facecolor=fill_color, edgecolor=outline_color)
        self.ax.add_patch(p)
        
    def draw_dot(self, size, pos, color):
        p = plt.Circle(pos, size, facecolor=color, edgecolor='None')
        self.ax.add_patch(p)



import pymunk as p
from pymunk.vec2d import Vec2d
import unittest

import pickle

class UnitTestShapeFilter(unittest.TestCase):
    def testInit(self):
        f = p.ShapeFilter()
        self.assertEqual(f.group, 0)
        self.assertEqual(f.categories, 0xffffffff)
        self.assertEqual(f.mask, 0xffffffff)

        f = p.ShapeFilter(1,2,3)
        self.assertEqual(f.group, 1)
        self.assertEqual(f.categories, 2)
        self.assertEqual(f.mask, 3)

    def testConstants(self):
        self.assertEqual(p.ShapeFilter.ALL_MASKS, 0xffffffff)
        self.assertEqual(p.ShapeFilter.ALL_CATEGORIES, 0xffffffff)

    def testEq(self):
        f1 = p.ShapeFilter(1,2,3)
        f2 = p.ShapeFilter(1,2,3)
        f3 = p.ShapeFilter(2,3,4)
        self.assertTrue(f1 == f2)
        self.assertTrue(f1 != f3)

    def testPickle(self):
        x = p.ShapeFilter(1,2,3)
        s = pickle.dumps(x, 2)
        actual = pickle.loads(s)
        self.assertEqual(x, actual)

class UnitTestContactPoint(unittest.TestCase):
    pass
    
class UnitTestContactPointSet(unittest.TestCase):
    pass



__version__ = "$Id$"
__docformat__ = "reStructuredText"

import sys
import warnings

from . import _chipmunk_cffi
cp = _chipmunk_cffi.lib
ffi = _chipmunk_cffi.ffi   

from .arbiter import Arbiter

class CollisionHandler(object):
    """A collision handler is a set of 4 function callbacks for the different
    collision events that Pymunk recognizes.

    Collision callbacks are closely associated with Arbiter objects. You
    should familiarize yourself with those as well.

    Note #1: Shapes tagged as sensors (Shape.sensor == true) never generate
    collisions that get processed, so collisions between sensors shapes and
    other shapes will never call the post_solve() callback. They still
    generate begin(), and separate() callbacks, and the pre_solve() callback
    is also called every frame even though there is no collision response.
    Note #2: pre_solve() callbacks are called before the sleeping algorithm
    runs. If an object falls asleep, its post_solve() callback won't be
    called until it's re-awoken.
    """
    def __init__(self, _handler, space, *args, **kwargs):
        """Initialize a CollisionHandler object from the Chipmunk equivalent
        struct and the Space.

        .. note::
            You should never need to create an instance of this class directly.
        """
        self._handler = _handler
        self._space = space
        self._begin = None
        self._begin_base = None # For pickle
        self._pre_solve = None
        self._pre_solve_base = None # For pickle
        self._post_solve = None
        self._post_solve_base = None # For pickle
        self._separate = None
        self._separate_base = None # For pickle

        self._data = {}
        
        
    def _get_data(self):
        return self._data
    data = property(_get_data, doc="""Data property that get passed on into the
    callbacks.

    data is a dictionary and you can not replace it, only fill it with data.
    
    Usefull if the callback needs some extra data to perform its function. 
    """)    

    def _set_begin(self, func):
        
        @ffi.callback("cpCollisionBeginFunc")
        def cf(_arb, _space, _):
            x = func(Arbiter(_arb, self._space), self._space, self._data)
            if isinstance(x,int):
                return x
            
            if sys.version_info[0] >= 3:
                func_name = func.__code__.co_name
                filename = func.__code__.co_filename
                lineno = func.__code__.co_firstlineno
            else:
                func_name = func.func_name
                filename = func.func_code.co_filename
                lineno = func.func_code.co_firstlineno

            warnings.warn_explicit(
                "Function '" + func_name + "' should return a bool to"
                " indicate if the collision should be processed or not when"
                " used as 'begin' or 'pre_solve' collision callback.",
                UserWarning, filename, lineno, func.__module__)
            return True
        
        self._begin = cf
        self._begin_base = func
        self._handler.beginFunc = cf

    def _get_begin(self):
        return self._begin

    begin = property(_get_begin, _set_begin,
        doc="""Two shapes just started touching for the first time this step.

        ``func(arbiter, space, data) -> bool``

        Return true from the callback to process the collision normally or
        false to cause pymunk to ignore the collision entirely. If you return
        false, the `pre_solve` and `post_solve` callbacks will never be run,
        but you will still recieve a separate event when the shapes stop
        overlapping.
        """)

    def _set_pre_solve(self, func):
        
        @ffi.callback("cpCollisionPreSolveFunc")
        def cf(_arb, _space, _data):
            x = func(Arbiter(_arb, self._space), self._space, self._data)
            if isinstance(x,int):
                return x
            
            if sys.version_info[0] >= 3:
                func_name = func.__code__.co_name
                filename = func.__code__.co_filename
                lineno = func.__code__.co_firstlineno
            else:
                func_name = func.func_name
                filename = func.func_code.co_filename
                lineno = func.func_code.co_firstlineno

            warnings.warn_explicit(
                "Function '" + func_name + "' should return a bool to"
                " indicate if the collision should be processed or not when"
                " used as 'begin' or 'pre_solve' collision callback.",
                UserWarning, filename, lineno, func.__module__)
            return True
        
        self._pre_solve = cf
        self._pre_solve_base = func
        self._handler.preSolveFunc = cf 
        
    def _get_pre_solve(self):
        return self._pre_solve

    pre_solve = property(_get_pre_solve, _set_pre_solve,
        doc="""Two shapes are touching during this step.

        ``func(arbiter, space, data) -> bool``

        Return false from the callback to make pymunk ignore the collision
        this step or true to process it normally. Additionally, you may
        override collision values using Arbiter.friction, Arbiter.elasticity
        or Arbiter.surfaceVelocity to provide custom friction, elasticity,
        or surface velocity values. See Arbiter for more info.
        """)

    def _set_post_solve(self, func):
    
        @ffi.callback("cpCollisionPostSolveFunc")
        def cf(_arb, _space, _data):
            func(Arbiter(_arb, self._space), self._space, self._data)
                
        self._post_solve = cf
        self._post_solve_base = func
        self._handler.postSolveFunc = cf

    def _get_post_solve(self):
        return self._post_solve

    post_solve = property(_get_post_solve, _set_post_solve,
        doc="""Two shapes are touching and their collision response has been
        processed.

        ``func(arbiter, space, data)``

        You can retrieve the collision impulse or kinetic energy at this
        time if you want to use it to calculate sound volumes or damage
        amounts. See Arbiter for more info.
        """)

    def _set_separate(self, func):
    
        @ffi.callback("cpCollisionSeparateFunc")
        def cf(_arb, _space, _data):
            func(Arbiter(_arb, self._space), self._space, self._data)
                
        self._separate = cf
        self._separate_base = func
        self._handler.separateFunc = cf

    def _get_separate(self):
        return self._separate

    separate = property(_get_separate, _set_separate,
        doc="""Two shapes have just stopped touching for the first time this
        step.

        ``func(arbiter, space, data)``

        To ensure that begin()/separate() are always called in balanced
        pairs, it will also be called when removing a shape while its in
        contact with something or when de-allocating the space.
        """)



import doctest
import unittest
import pymunk
import pkgutil
import sys


def load_tests(loader, tests, ignore):
    for importer, modname, ispkg in pkgutil.iter_modules(pymunk.__path__):
        try:
            tests.addTests(doctest.DocTestSuite("pymunk." + modname))
        except:
            print("Skipping " + modname)
    tests.addTests(doctest.DocTestSuite(pymunk))
    return tests


if __name__ == '__main__':
    print("running doctests")
    suite = unittest.TestSuite()  
    load_tests(None, suite, None)
    res = unittest.TextTestRunner().run(suite)
    sys.exit(not res.wasSuccessful())
    
    


import pymunk as p
from pymunk.vec2d import Vec2d
import unittest
import pickle

class UnitTestTransform(unittest.TestCase):
    def testInit(self):
        
        t = p.Transform(1,2,3,4,5,6)
        self.assertEqual(t.a, 1)
        self.assertEqual(t.b, 2)
        self.assertEqual(t.c, 3)
        self.assertEqual(t.d, 4)
        self.assertEqual(t.tx, 5)
        self.assertEqual(t.ty, 6)
        self.assertEqual(str(t), "Transform(a=1, b=2, c=3, d=4, tx=5, ty=6)")

        t = p.Transform(b=4, ty=2)
        self.assertEqual(t.a, 1)
        self.assertEqual(t.b, 4)
        self.assertEqual(t.c, 0)
        self.assertEqual(t.d, 1)
        self.assertEqual(t.tx, 0)
        self.assertEqual(t.ty, 2)

    def testIdentity(self):
        t = p.Transform.identity()
        self.assertEqual(t.a, 1)
        self.assertEqual(t.b, 0)
        self.assertEqual(t.c, 0)
        self.assertEqual(t.d, 1)
        self.assertEqual(t.tx, 0)
        self.assertEqual(t.ty, 0)

    def testPickle(self):
        x = p.Transform.identity()
        s = pickle.dumps(x, 2)
        actual = pickle.loads(s)
        self.assertEqual(x, actual)
